#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

__all__ = [
    "config_testone",
    "config_testone_xx",
    "config_modules",
    "config_modules_2"
]


config_testone = {
    "name": "testone",
    "command": "test01",
    "desc": "the first test",
    "workers": [
        {
            "path": "testdata.app.commands.prepact",
            "classname": "Worker01"
        },
        {
            "path": "testdata.app.commands.runact",
            "classname": "Worker02"
        }
    ]
}


config_testone_xx = {
    "name": "testone",
    "command": "test01",
    "desc": "the first test",
    "workers": [
        {
            "path": "testdata.appx.commands.prepact",
            "classname": "Worker01"
        },
        {
            "path": "testdata.app.commands.runact",
            "classname": "Worker02"
        }
    ]
}


config_modules = [
    {
        "name": "testone",
        "command": "test01",
        "desc": "the first test",
        "workers": [
            {
                "path": "testdata.app.commands.prepact",
                "classname": "Worker01"
            },
            {
                "path": "testdata.app.commands.runact",
                "classname": "Worker02"
            }
        ]
    },
    {
        "name": "testtwo",
        "command": "test02",
        "desc": "the second test",
        "workers": [
            {
                "path": "testdata.app.commands.prepact",
                "classname": "Worker01"
            }
        ]
    },
    {
        "name": "testthree",
        "command": "test03",
        "desc": "the third test",
        "workers": [
            {
                "path": "testdata.app.commands.prepact2",
                "classname": "Worker03"
            }
        ]
    }
]


config_modules_2 = [
    {
        "name": "testone",
        "command": "test01",
        "desc": "the first test",
        "workers": [
            {
                "path": "testdata.appx.commands.prepact",
                "classname": "Worker01"
            },
            {
                "path": "testdata.appx.commands.runact",
                "classname": "Worker02"
            }
        ]
    },
    {
        "name": "testtwo",
        "command": "test02",
        "desc": "the second test",
        "workers": [
            {
                "path": "testdata.appx.commands.prepact",
                "classname": "Worker01"
            }
        ]
    }
]



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import gettext

from typing import Optional, Any, List

__all__ = [
    "Domain",
]

lang_domain = None


class Domain(object):

    def __init__(self, localedir: str, domain: str, use_dummy: bool, ignore, used_lang: str):
        self.localedir: str = localedir
        self.domain: str = domain
        self.gettext: Optional[gettext.translation] = None
        self.lang = None
        self.is_set: bool = False
        self.ignore = ignore
        self.callback: List[Any] = []
        self.used_lang: str = used_lang
        self.use_dummy: bool = use_dummy
        return

    @staticmethod
    def _dummy(text: str):
        return text

    def create(self):
        if self.is_set is True:
            return

        if self.ignore is not None:
            self.use_dummy = True

        if self.use_dummy is True:
            self.lang = self._dummy
            self.is_set = True
            return

        used_catalog = [
            self.used_lang
        ]

        self.gettext = gettext.translation(self.domain, localedir=self.localedir, languages=used_catalog)
        self.gettext.install()

        self.lang = self.gettext.gettext
        self.is_set = True
        return

    def load(self):
        for _callback in self.callback:
            _callback(self.lang)
        return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

from typing import List, Optional

from bbutil.app.module import Module

__all__ = [
    "ModuleManager"
]


class ModuleManager(object):

    def __init__(self):
        self.modules: List[Module] = []
        self.commands: List[str] = []
        return

    def init(self, config: list) -> bool:

        for _config in config:
            _module = Module(**_config)

            self.commands.append(_module.command)
            self.modules.append(_module)
        return True

    def has_command(self, command: str) -> bool:
        if command in self.commands:
            return True
        return False

    def get_command(self, command_id: str) -> Optional[Module]:

        for _module in self.modules:
            if _module.command == command_id:
                return _module
        return None



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import unittest

import unittest.mock as mock

from bbutil.app.manager import ModuleManager

from tests.helper import set_log, reset_module
from tests.helper.module import config_modules


__all__ = [
    "TestManager"
]

oserror = OSError("Something strange did happen!")
mock_oserror = mock.Mock(side_effect=oserror)
mock_remove = mock.Mock()


class TestManager(unittest.TestCase):
    """Testing class for locking module."""

    def setUp(self):
        set_log()
        reset_module()
        return

    def test_init_01(self):
        _module = ModuleManager()
        _check1 = _module.init(config_modules)

        _command1 = _module.get_command("test01")
        _command2 = _module.get_command("test01xx")
        _check2 = _module.has_command("test01")
        _check3 = _module.has_command("test01xx")

        self.assertTrue(_check1)
        self.assertTrue(_check2)
        self.assertFalse(_check3)
        self.assertIsNotNone(_command1)
        self.assertIsNone(_command2)
        self.assertEqual(len(_module.commands), 3)
        self.assertEqual(len(_module.modules), 3)
        return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

from dataclasses import dataclass
from typing import Optional

from unittest.mock import Mock

from bbutil.database import Types, Table, Database

__all__ = [
    "TestData",
    "TestData2"
]


@dataclass
class TestData(Database):

    table01: Optional[Table] = None
    table02: Optional[Table] = None
    prepare_fail: bool = False
    mock_connection: Optional[Mock] = None

    def init(self):
        self.name = "Testos"
        return

    def prepare(self, **kwargs) -> bool:
        if self.prepare_fail is True:
            return False

        if self.mock_connection is not None:
            self.sqlite.connection = self.mock_connection

        _table = Table(name="tester01", sqlite=self.sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)
        self.tables.append(_table)
        self.table01 = _table

        _table = Table(name="tester02", sqlite=self.sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="category", data_type=Types.string, keyword=True)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)
        self.tables.append(_table)
        self.table02 = _table
        return True

    def clear_data(self):
        return


@dataclass
class TestData2(Database):

    table01: Optional[Table] = None
    table02: Optional[Table] = None
    prepare_fail: bool = False
    mock_connection: Optional[Mock] = None

    def init(self):
        self.name = "Testos"
        return

    def prepare(self, **kwargs) -> bool:
        if self.prepare_fail is True:
            return False

        if self.mock_connection is not None:
            self.sqlite.connection = self.mock_connection

        _table = Table(name="tester01", sqlite=self.sqlite, version=1)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)
        self.tables.append(_table)
        self.table01 = _table

        _table = Table(name="tester02", sqlite=self.sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="category", data_type=Types.string, keyword=True)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)
        self.tables.append(_table)
        self.table02 = _table
        return True

    def clear_data(self):
        return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

from dataclasses import dataclass, field
from typing import Optional, List

import bbutil
from bbutil.utils import get_attribute
from bbutil.worker import Worker

__all__ = [
    "Module"
]


@dataclass
class _Worker(object):

    path: str = ""
    classname: str = ""


class Module(object):

    def __init__(self, **kwargs):
        self.name: str = ""
        self.command: str = ""
        self.desc: str = ""
        self.workers: List[Worker] = []
        self._workers: List[_Worker] = []

        _value = kwargs.get("name", None)
        if _value is not None:
            self.name = _value

        _value = kwargs.get("command", None)
        if _value is not None:
            self.command = _value

        _value = kwargs.get("desc", None)
        if _value is not None:
            self.desc = _value

        _value = kwargs.get("workers", None)
        if _value is not None:
            self._parse_workers(_value)
        return

    def _parse_workers(self, workers: list):
        for _conifg in workers:

            # noinspection PyArgumentList
            _worker = _Worker(**_conifg)

            self._workers.append(_worker)
        return

    @property
    def count(self) -> int:
        return len(self.workers)

    def get_worker(self, worker_id: str) -> Optional[Worker]:
        for _worker in self.workers:
            if _worker.id == worker_id:
                return _worker
        return None

    def load(self) -> bool:

        for item in self._workers:

            try:
                c = get_attribute(item.path, item.classname)
            except ImportError as e:
                bbutil.log.error("Unable to get worker class!")
                bbutil.log.exception(e)
                return False

            _worker = c()
            self.workers.append(_worker)
        return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import abc
import threading
import time

from abc import ABCMeta
from dataclasses import dataclass
from typing import Optional

import bbutil
from bbutil.worker.callback import Callback

__all__ = [
    "Worker",

    "callback"
]


@dataclass
class Worker(metaclass=ABCMeta):

    abort: bool = False
    interval: float = 0.01

    _id: str = ""
    _callback: Optional[Callback] = None
    _error: bool = False
    _running: bool = True

    def __post_init__(self):
        self.init()
        return

    @property
    def id(self) -> str:
        return self._id

    @property
    def error(self) -> bool:
        return self._error

    def set_id(self, new_id: str):
        self._id = new_id
        return

    @abc.abstractmethod
    def init(self):
        pass

    @abc.abstractmethod
    def prepare(self) -> bool:
        pass

    @abc.abstractmethod
    def run(self) -> bool:
        pass

    @abc.abstractmethod
    def close(self) -> bool:
        pass

    def set_callback(self, **kwargs):
        if self._callback is None:
            self._callback = Callback()

        self._callback.set_callback(**kwargs)
        return

    def _do_step(self, step: str, function, callback_func):
        if self.abort is True:
            self._running = False
            self.abort = False
            self._callback.do_abort()
            bbutil.log.warn(self._id, "Abort {0:s}".format(step))
            return

        callback_func()

        _check = function()
        if _check is False:
            self._error = True
            bbutil.log.error("{0:s}: {1:s} failed!".format(self.id, step))

        if self._error is True:
            self._running = False
        return

    def _execute(self):
        if self._callback is None:
            self._callback = Callback()

        self._running = True
        self._callback.do_start()

        bbutil.log.inform(self.id, "Execute")

        self._do_step("prepare", self.prepare, self._callback.do_prepare)
        if self._running is False:
            self._callback.do_stop()
            return

        self._do_step("run", self.run, self._callback.do_run)
        if self._running is False:
            self._callback.do_stop()
            return

        self._do_step("close", self.close, self._callback.do_close)
        if self._running is False:
            self._callback.do_stop()
            return

        self._callback.do_stop()
        self._running = False
        return

    def start(self):
        _t = threading.Thread(target=self._execute)
        _t.start()
        return

    @property
    def is_running(self):
        return self._running

    def wait(self):
        _run = True

        while _run is True:
            time.sleep(self.interval)

            if self._running is False:
                _run = False
        return

    def execute(self) -> bool:
        self._execute()

        if self._error is True:
            return False

        return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

from typing import Optional
from bbutil.logging import Logging
from bbutil.app.config import Config
from bbutil.app.manager import ModuleManager

__all__ = [
    "app",
    "database",
    "lang",
    "logging",
    "worker",

    "data",
    "execute",
    "file",
    "setup",
    "utils",

    "log",
    "config",
    "module",

    "set_log",
    "set_config",
    "set_module"
]


#: package name
__name__ = "bbutils"

#: package author
__author__ = "Kai Raphahn"

#: email of package maintainer
__email__ = "kai.raphahn@laburec.de"

#: copyright year
__year__ = 2024

#: package copyright
__copyright__ = "Copyright (C) {0:d}, {1:s} <{2:s}>".format(__year__, __author__, __email__)

#: package description
__description__ = "Small collection of stuff for all my other python projects (including logging)."

#: package license
__license__ = "Apache License, Version 2.0"

#: package credits
__credits__ = [__author__]

#: version milestone
__milestone__ = 0

#: version major
__major__ = 7

#: version minor
__minor__ = 0

#: version patch
__patch__ = 0

#: package version
__version__ = "{0:d}.{1:d}.{2:d}.{3:d}".format(__milestone__, __major__, __minor__, __patch__)

#: package maintainer
__maintainer__ = __author__


log: Optional[Logging] = None
config: Optional[Config] = None
module: Optional[ModuleManager] = None


def set_log(item: Logging):
    global log
    log = item
    return


def set_config(item: Config):
    global config
    config = item
    return


def set_module(item: ModuleManager):
    global module
    module = item
    return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import os.path

from typing import Optional

from bbutil.utils import full_path, get_attribute
from bbutil.logging import Logging

__all__ = [
    "PythonFile"
]

log: Optional[Logging] = None


def set_log(logging: Logging):
    global log
    log = logging
    return


class PythonFile(object):

    def __repr__(self):
        return self._basename

    @property
    def basename(self) -> str:
        return self._basename

    def __init__(self,
                 package_path: str,
                 filename: str,
                 module: str,
                 module_filter: str = "",
                 locales: str = ".locales"):

        self._package_path: str = package_path
        self._basename: str = os.path.basename(filename).replace(".py", "")
        self._module: str = module
        self._module_filter: str = module_filter
        self._locales: str = locales

        self.fullname: str = filename
        self.domain: str = ""
        self.classname: str = ""
        self.pot: str = ""
        self.path: str = ""
        return

    def create(self) -> bool:

        _root = full_path("{0:s}/".format(self._package_path))
        _root = "{0:s}{1:s}".format(_root, os.path.sep)
        _filename = self.fullname.replace(_root, "")

        module_path = _filename.replace(".py", "")
        module_path = module_path.replace(os.path.sep, ".")

        if "__init__" in module_path:
            module_path = module_path.replace(".__init__", "")

        if self._module == "":
            self.classname = module_path
        else:
            self.classname = "{0:s}.{1:s}".format(self._module, module_path)

        _info = "{0:s}: {1:s}".format(self._package_path, _filename)
        log.debug3("Parse", _info)

        if self._module_filter not in self.classname:
            return False

        try:
            lang = get_attribute(self.classname, "lang_domain")
        except ImportError as e:
            lang = None
            log.error("Unable to open module: {0:s}".format(self.classname))
            log.exception(e)

        if lang is None:
            return False

        _logtext = "{0:s}: {1:s}".format(lang, self.classname)

        log.inform("Add", _logtext)
        self.domain = lang
        _pot = "{0:s}/{1:s}/_{2:s}.pot".format(self._locales, self.domain, self.classname)
        _path = "{0:s}/{1:s}".format(self._locales, self.domain)
        self.pot = full_path(_pot)
        self.path = full_path(_path)
        return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

from bbutil.app.module import Module
from bbutil.app.config import Config
from bbutil.app.manager import ModuleManager
from bbutil.app.console import Console

__all__ = [
    "config",
    "console",
    "manager",
    "module",

    "Module",
    "Config",
    "ModuleManager",
    "Console"
]



#!/usr/bin/python3
# coding=utf-8
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#


from dataclasses import dataclass
from enum import Enum
from typing import Any

__all__ = [
    "select_interval",

    "DataType",
    "Types",
    "Data",
    "Column"
]


def select_interval(count) -> int:
    _interval = 1

    if count > 1000:
        _interval = 10

    if count > 10000:
        _interval = 100

    if count > 100000:
        _interval = 1000
    return _interval


@dataclass
class DataType(object):

    type: str = ""
    value: Any = None


class Types(Enum):

    none = DataType(type="NULL", value=None)
    json = DataType(type="JSON", value={})
    bool = DataType(type="BOOLEAN", value=False)
    datetime = DataType(type="timestamp", value=None)
    integer = DataType(type="INTEGER", value=0)
    biginteger = DataType(type="BIGINT", value=0)
    float = DataType(type="REAL", value=0.0)
    string = DataType(type="TEXT", value="")
    bytes = DataType(type="BLOB", value=b"")
    list = DataType(type="BLOB", value=b"")


class Data(object):

    def __init__(self, keys: list, values: list):
        for (key, value) in zip(keys, values):
            self.__dict__[key] = value
        return


@dataclass
class Column(object):

    name: str = ""
    primarykey: bool = False
    unique: bool = False
    type: Types = Types.none

    @property
    def create(self) -> str:
        _datatype: DataType = self.type.value

        if self.primarykey is True:
            _ret = '"{0:s}" INTEGER PRIMARY KEY AUTOINCREMENT'.format(self.name)
            return _ret

        _ret = '"{0:s}" {1:s}'.format(self.name, _datatype.type)
        return _ret



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import sqlite3
import os
import unittest
import unittest.mock as mock

from bbutil.database import SQLite, Table, Types
from bbutil.utils import full_path

from tests.helper.sqlite import get_sqlite_operational_error, get_sqlite_integrity_error, get_sqlite_return_false
from tests.helper.sqlite import (get_table_01, get_data_01, get_data_02, get_data_03, get_data_04, get_data_05,
                                 get_data_06, get_data_07, get_data_08)

from tests.helper import get_sqlite, set_log, copy_sqlite

__all__ = [
    "TestSQLite"
]


def _get_sqlite_version(major_diff: int, minor_diff: int, patch_diff: int = 0) -> tuple:
    _info = sqlite3.sqlite_version_info

    _major = _info[0] + major_diff
    _minor = _info[0] + minor_diff
    _patch = _info[0] + patch_diff

    _version = (_major, 0, 0)
    return _version


class TestSQLite(unittest.TestCase):
    """Testing class for locking module."""

    @staticmethod
    def check_minmal_version(major: int, minor: int, patch: int = 0) -> bool:
        _info = sqlite3.sqlite_version_info

        _major = int(_info[0])
        _minor = int(_info[1])
        _patch = int(_info[2])

        _version = "{0:d}.{1:d}.{2:d}".format(major, minor, patch)
        _error1 = "Version check failed!"
        _error2 = "Minimum version requierd is {0:s}, current version is {1:s}!".format(_version,
                                                                                        sqlite3.sqlite_version)

        if _major < major:
            print(_error1)
            print(_error2)
            return False

        if _minor < minor:
            print(_error1)
            print(_error2)
            return False

        return True

    def setUp(self):
        set_log()
        return

    @staticmethod
    def _clean(sqlite: SQLite):
        _con = None

        if sqlite.manager is not None:
            _con = sqlite.manager.connection

        if _con is not None:
            _con.close()

        if os.path.exists(sqlite.filename) is True:
            os.remove(sqlite.filename)
        return

    def test_prepare_01(self):
        _testfile = full_path("{0:s}/test.sqlite".format(os.getcwd()))
        _name = "Test"

        if os.path.exists(_testfile) is True:
            os.remove(_testfile)

        _sqlite = SQLite(filename=_testfile, name="Test")

        self.assertIsNone(_sqlite.manager)

        _sqlite.prepare()
        _sqlite.prepare()

        self.assertEqual(_sqlite.name, _name)
        self.assertEqual(_sqlite.filename, _testfile)
        self.assertIsNotNone(_sqlite.manager)

        self._clean(_sqlite)
        return

    def test_check_01(self):
        _sqlite = get_sqlite(filename="test.sqlite")
        _sqlite.prepare()

        _check = _sqlite.check("Test")
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    def test_check_02(self):
        _sqlite = get_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _check = _sqlite.check("tester01")
        self.assertTrue(_check)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_operational_error())
    def test_check_03(self):
        _sqlite = get_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _check = _sqlite.check("tester01")
        self.assertFalse(_check)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_operational_error())
    def test_check_04(self):
        _sqlite = get_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _check = _sqlite.check("tester01")
        self.assertFalse(_check)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.connect', new=get_sqlite_return_false())
    def test_check_05(self):
        _sqlite = get_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _check = _sqlite.check("tester01")
        self.assertFalse(_check)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.release', new=get_sqlite_return_false())
    def test_check_06(self):
        _sqlite = get_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _check = _sqlite.check("tester01")
        self.assertFalse(_check)
        return

    def test_check_minmal_version_01(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _major, _minor, patch = sqlite3.sqlite_version_info

        _check = _sqlite.check_minmal_version(_major, _minor, patch)
        self.assertTrue(_check)
        self._clean(_sqlite)
        return

    def test_check_minmal_version_02(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _major, _minor, patch = sqlite3.sqlite_version_info

        _check = _sqlite.check_minmal_version(_major + 1, _minor, patch)
        self.assertFalse(_check)
        self._clean(_sqlite)
        return

    def test_check_minmal_version_03(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _major, _minor, patch = sqlite3.sqlite_version_info

        _check = _sqlite.check_minmal_version(_major, _minor + 1, patch)
        self.assertFalse(_check)
        self._clean(_sqlite)
        return

    def test_check_minmal_version_04(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _major, _minor, patch = sqlite3.sqlite_version_info

        _check = _sqlite.check_minmal_version(_major, _minor, patch + 1)
        self.assertFalse(_check)
        self._clean(_sqlite)
        return

    def test_check_minmal_version_05(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _major, _minor, patch = sqlite3.sqlite_version_info

        _check = _sqlite.check_minmal_version(_major - 1, _minor, patch)
        self.assertTrue(_check)
        self._clean(_sqlite)
        return

    def test_check_minmal_version_06(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _major, _minor, patch = sqlite3.sqlite_version_info

        _check = _sqlite.check_minmal_version(_major, _minor - 1, patch)
        self.assertTrue(_check)
        self._clean(_sqlite)
        return

    def test_check_minmal_version_07(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _major, _minor, patch = sqlite3.sqlite_version_info

        _check = _sqlite.check_minmal_version(_major, _minor, patch - 1)
        self.assertTrue(_check)
        self._clean(_sqlite)
        return

    def test_count_01(self):
        _sqlite = get_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _count = _sqlite.count("tester01")
        self.assertEqual(_count, 0)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_operational_error())
    def test_count_02(self):
        _sqlite = get_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _count = _sqlite.count("tester01")
        self.assertEqual(_count, -1)
        return

    def test_count_03(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()

        _count = _sqlite.count("tester01")
        self.assertEqual(_count, 6)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.connect', new=get_sqlite_return_false())
    def test_count_04(self):
        _sqlite = get_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _count = _sqlite.count("tester01")
        self.assertEqual(_count, -1)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.release', new=get_sqlite_return_false())
    def test_count_05(self):
        _sqlite = get_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _count = _sqlite.count("tester01")
        self.assertEqual(_count, -1)
        return

    def test_check_table_01(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()

        _check = _sqlite.check_table("tester01")
        self.assertTrue(_check)
        return

    def test_check_table_02(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()

        _check = _sqlite.check_table("tester01CXXX")
        self.assertFalse(_check)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.connect', new=get_sqlite_return_false())
    def test_check_table_03(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()

        _check = _sqlite.check_table("tester01")
        self.assertFalse(_check)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.release', new=get_sqlite_return_false())
    def test_check_table_04(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()

        _check = _sqlite.check_table("tester01")
        self.assertFalse(_check)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_operational_error())
    def test_check_table_05(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()

        _check = _sqlite.check_table("tester01")
        self.assertFalse(_check)
        return

    def test_create_table_01(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _sqlite.prepare()
        _table = get_table_01(_sqlite)

        _check = _sqlite.create_table(_table.name, _table.column_list, _table.unique_list)
        self.assertTrue(_check)

        _check = _sqlite.check("tester01")
        self.assertTrue(_check)

        _check = _sqlite.check_table(_table.name)
        self.assertTrue(_check)
        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.connect', new=get_sqlite_return_false())
    def test_create_table_02(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _sqlite.prepare()
        _table = get_table_01(_sqlite)

        _check = _sqlite.create_table(_table.name, _table.column_list, _table.unique_list)
        self.assertFalse(_check)
        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.release', new=get_sqlite_return_false())
    def test_create_table_03(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _sqlite.prepare()
        _table = get_table_01(_sqlite)

        _check = _sqlite.create_table(_table.name, _table.column_list, _table.unique_list)
        self.assertFalse(_check)
        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.release', new=get_sqlite_return_false())
    def test_create_table_04(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()
        _table = get_table_01(_sqlite)

        _check = _sqlite.create_table(_table.name, _table.column_list, _table.unique_list)
        self.assertFalse(_check)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.commit', new=get_sqlite_return_false())
    def test_create_table_05(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _sqlite.prepare()
        _table = get_table_01(_sqlite)

        _check = _sqlite.create_table(_table.name, _table.column_list, _table.unique_list)
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    def test_create_table_06(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _sqlite.prepare()
        _table = get_table_01(_sqlite)

        with mock.patch('sqlite3.connect', new=get_sqlite_operational_error()):
            _check = _sqlite.create_table(_table.name, _table.column_list, _table.unique_list)
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    def test_get_scheme_01(self):
        _sqlite = get_sqlite(filename="test_database.sqlite", path="testdata/database")

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, primarykey=True)
        _table.add_column(name="use_test", data_type=Types.string)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _sqlite.prepare()

        _data = _sqlite.get_scheme(_table.name)

        self.assertIsNotNone(_data)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.connect', new=get_sqlite_return_false())
    def test_get_scheme_02(self):
        _sqlite = get_sqlite(filename="test_database.sqlite", path="testdata/database")
        _sqlite.prepare()

        _data = _sqlite.get_scheme("test")

        self.assertIsNone(_data)
        return

    def test_get_scheme_03(self):
        _sqlite = get_sqlite(filename="test_database.sqlite", path="testdata/database")
        _sqlite.prepare()

        _data = _sqlite.get_scheme("test")

        self.assertIsNone(_data)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_operational_error())
    def test_get_scheme_04(self):
        _sqlite = get_sqlite(filename="test_database.sqlite", path="testdata/database")
        _sqlite.prepare()

        _data = _sqlite.get_scheme("test")

        self.assertIsNone(_data)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.release', new=get_sqlite_return_false())
    def test_get_scheme_05(self):
        _sqlite = get_sqlite(filename="test_database.sqlite", path="testdata/database")
        _sqlite.prepare()

        _data = _sqlite.get_scheme("test")

        self.assertIsNone(_data)
        return

    def test_add_columns_01(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _sqlite.create_table(_table.name, _table.column_list, _table.unique_list)
        self.assertTrue(_check)

        _scheme1 = _sqlite.get_scheme(_table.name)

        _table.add_column(name="new1", data_type=Types.string)
        _table.add_column(name="new2", data_type=Types.string)
        _column1 = _table.get_column("new1")
        _column2 = _table.get_column("new2")

        self.assertIsNotNone(_column1)
        self.assertIsNotNone(_column2)

        _column_list = [
            _column1.create,
            _column2.create
        ]

        _check = _sqlite.add_columns(_table.name, _column_list)
        self.assertTrue(_check)

        _scheme2 = _sqlite.get_scheme(_table.name)
        _test_scheme1 = [
            ('testid', 'INTEGER'),
            ('use_test', 'BOOLEAN'),
            ('testname', 'TEXT'),
            ('path', 'TEXT')
        ]

        _test_scheme2 = [
            ('testid', 'INTEGER'),
            ('use_test', 'BOOLEAN'),
            ('testname', 'TEXT'),
            ('path', 'TEXT'),
            ('new1', 'TEXT'),
            ('new2', 'TEXT')
        ]

        self.assertListEqual(_scheme1, _test_scheme1)
        self.assertListEqual(_scheme2, _test_scheme2)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.connect', new=get_sqlite_return_false())
    def test_add_columns_02(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)
        _table.add_column(name="new1", data_type=Types.string)

        _column1 = _table.get_column("new1")

        self.assertIsNotNone(_column1)

        _column_list = [
            _column1.create,
        ]

        _check = _sqlite.add_columns(_table.name, _column_list)
        self.assertFalse(_check)
        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.release', new=get_sqlite_return_false())
    def test_add_columns_03(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _table.add_column(name="new1", data_type=Types.string)
        _table.add_column(name="new2", data_type=Types.string)
        _column1 = _table.get_column("new1")
        _column2 = _table.get_column("new2")

        self.assertIsNotNone(_column1)
        self.assertIsNotNone(_column2)

        _column_list = [
            _column1.create,
            _column2.create
        ]

        _check = _sqlite.add_columns(_table.name, _column_list)
        self.assertFalse(_check)
        self._clean(_sqlite)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_operational_error())
    def test_add_columns_04(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _table.add_column(name="new1", data_type=Types.string)
        _table.add_column(name="new2", data_type=Types.string)
        _column1 = _table.get_column("new1")
        _column2 = _table.get_column("new2")

        self.assertIsNotNone(_column1)
        self.assertIsNotNone(_column2)

        _column_list = [
            _column1.create,
            _column2.create
        ]

        _check = _sqlite.add_columns(_table.name, _column_list)
        self.assertFalse(_check)
        self._clean(_sqlite)
        return

    def _test_drop_columns_01_simul(self):
        return

    def test_drop_columns_01(self):
        # The drop functionality needs a minimal version to work
        _check = self.check_minmal_version(3, 35, 0)
        if _check is False:
            return

        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)
        _table.add_column(name="new1", data_type=Types.string)
        _table.add_column(name="new2", data_type=Types.string)

        _check = _sqlite.create_table(_table.name, _table.column_list, _table.unique_list)
        self.assertTrue(_check)

        _scheme1 = _sqlite.get_scheme(_table.name)
        _test_scheme1 = [
            ('testid', 'INTEGER'),
            ('use_test', 'BOOLEAN'),
            ('testname', 'TEXT'),
            ('path', 'TEXT'),
            ('new1', 'TEXT'),
            ('new2', 'TEXT')
        ]
        self.assertListEqual(_scheme1, _test_scheme1)

        _table.drop_columns.append("new1")
        _table.drop_columns.append("new2")

        _check = _sqlite.drop_columns(_table.name, _table.drop_columns)
        self.assertTrue(_check)

        _scheme2 = _sqlite.get_scheme(_table.name)
        _test_scheme2 = [
            ('testid', 'INTEGER'),
            ('use_test', 'BOOLEAN'),
            ('testname', 'TEXT'),
            ('path', 'TEXT')
        ]
        self.assertListEqual(_scheme2, _test_scheme2)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.connect', new=get_sqlite_return_false())
    def test_drop_columns_02(self):
        # The drop functionality needs a minimal version to work
        _check = self.check_minmal_version(3, 35, 0)
        if _check is False:
            return

        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)
        _table.add_column(name="new1", data_type=Types.string)
        _table.add_column(name="new2", data_type=Types.string)

        _check = _sqlite.drop_columns(_table.name, _table.drop_columns)
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_operational_error())
    def test_drop_columns_03(self):
        # The drop functionality needs a minimal version to work
        _check = self.check_minmal_version(3, 35, 0)
        if _check is False:
            return

        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _sqlite.drop_columns(_table.name, ["path"])
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.release', new=get_sqlite_return_false())
    def test_drop_columns_04(self):
        # The drop functionality needs a minimal version to work
        _check = self.check_minmal_version(3, 35, 0)
        if _check is False:
            return

        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _sqlite.drop_columns(_table.name, ["path"])
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('sqlite3.sqlite_version_info', new=(3, 20, 0))
    def test_drop_columns_05(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _sqlite.drop_columns(_table.name, ["path"])
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    def test_rename_table_01(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _sqlite.rename_table(_table.name, "newtester01")
        self.assertTrue(_check)

        _scheme = _sqlite.get_scheme("newtester01")
        _test_scheme = [
            ('testid', 'INTEGER'),
            ('use_test', 'BOOLEAN'),
            ('testname', 'TEXT'),
            ('path', 'TEXT')
        ]
        self.assertListEqual(_scheme, _test_scheme)
        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.connect', new=get_sqlite_return_false())
    def test_rename_table_02(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _sqlite.rename_table(_table.name, "newtester01")
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.release', new=get_sqlite_return_false())
    def test_rename_table_03(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _sqlite.rename_table(_table.name, "newtester01")
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_operational_error())
    def test_rename_table_04(self):
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _sqlite.rename_table(_table.name, "newtester01")
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    def test_rename_column_01(self):
        # The drop functionality needs a minimal version to work
        _check = self.check_minmal_version(3, 25, 0)
        if _check is False:
            return

        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _sqlite.rename_column(_table.name, "use_test", "use_test_new")
        self.assertTrue(_check)

        _scheme = _sqlite.get_scheme(_table.name)
        _test_scheme = [
            ('testid', 'INTEGER'),
            ('use_test_new', 'BOOLEAN'),
            ('testname', 'TEXT'),
            ('path', 'TEXT')
        ]
        self.assertListEqual(_scheme, _test_scheme)
        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.connect', new=get_sqlite_return_false())
    def test_rename_column_02(self):
        # The drop functionality needs a minimal version to work
        _check = self.check_minmal_version(3, 25, 0)
        if _check is False:
            return

        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _sqlite.rename_column(_table.name, "use_test", "use_test_new")
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.release', new=get_sqlite_return_false())
    def test_rename_column_03(self):
        # The drop functionality needs a minimal version to work
        _check = self.check_minmal_version(3, 25, 0)
        if _check is False:
            return

        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _sqlite.rename_column(_table.name, "use_test", "use_test_new")
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_operational_error())
    def test_rename_column_04(self):
        # The drop functionality needs a minimal version to work
        _check = self.check_minmal_version(3, 25, 0)
        if _check is False:
            return

        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _sqlite.rename_column(_table.name, "use_test", "use_test_new")
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('sqlite3.sqlite_version_info', new=(3, 20, 0))
    def test_rename_column_05(self):

        # sqlite3.sqlite_version_info
        _sqlite = copy_sqlite(filename="test_add_columns.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = Table(name="tester01", sqlite=_sqlite)
        _table.add_column(name="testid", data_type=Types.integer, unique=True)
        _table.add_column(name="use_test", data_type=Types.bool)
        _table.add_column(name="testname", data_type=Types.string)
        _table.add_column(name="path", data_type=Types.string)

        _check = _sqlite.rename_column(_table.name, "use_test", "use_test_new")
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    def test_insert_01(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        _check = _sqlite.create_table(_table.name, _table.column_list, _table.unique_list)
        self.assertTrue(_check)

        _data = get_data_01()
        count2 = _sqlite.insert(_table.name, _table.names, _data)
        count3 = _sqlite.count(_table.name)

        self.assertEqual(count2, 1)
        self.assertEqual(count3, 1)
        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.connect', new=get_sqlite_return_false())
    def test_insert_02(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        _data = get_data_01()
        count = _sqlite.insert(_table.name, _table.names, _data)
        self.assertEqual(count, -1)
        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.release', new=get_sqlite_return_false())
    def test_insert_03(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _data = get_data_01()

        count = _sqlite.insert(_table.name, _table.names, _data)

        self.assertEqual(count, -1)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.commit', new=get_sqlite_return_false())
    def test_insert_04(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _data = get_data_01()

        count = _sqlite.insert(_table.name, _table.names, _data)

        self.assertEqual(count, -1)

        self._clean(_sqlite)
        return

    def test_insert_05(self):
        _sqlite = get_sqlite(filename="test.sqlite", clean=True)
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        _check = _sqlite.create_table(_table.name, _table.column_list, _table.unique_list)
        self.assertTrue(_check)

        _data = get_data_02()
        count2 = _sqlite.insert(_table.name, _table.names, _data)

        _con = _sqlite.manager.connection
        _sqlite.manager.reset()

        count3 = _sqlite.count(_table.name)

        self.assertEqual(count2, -1)
        self.assertEqual(count3, 0)

        if _con is not None:
            _con.close()

        if os.path.exists(_sqlite.filename) is True:
            os.remove(_sqlite.filename)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_operational_error())
    def test_insert_06(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _data = get_data_01()

        count = _sqlite.insert(_table.name, _table.names, _data)
        self.assertEqual(count, -1)

        self._clean(_sqlite)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_integrity_error())
    def test_insert_07(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _data = get_data_01()

        count = _sqlite.insert(_table.name, _table.names, _data)
        self.assertEqual(count, -1)

        self._clean(_sqlite)
        return

    def test_insert_08(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _data = get_data_03()

        count = _sqlite.insert(_table.name, _table.names, _data)
        self.assertEqual(count, -1)

        self._clean(_sqlite)
        return

    def test_insert_09(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _data = get_data_04()

        count = _sqlite.insert(_table.name, _table.names, _data)
        self.assertEqual(count, -1)

        self._clean(_sqlite)
        return

    def test_insert_10(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _data = get_data_05()

        count = _sqlite.insert(_table.name, _table.names, _data)
        self.assertEqual(count, 6)

        self._clean(_sqlite)
        return

    def test_insert_11(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _data = get_data_06()

        count = _sqlite.insert(_table.name, _table.names, _data)
        self.assertEqual(count, -1)

        self._clean(_sqlite)
        return

    @staticmethod
    def _fill_bulk(table: Table, max_range: int = 0):
        _range = range(0, max_range)
        _width = len(str(max_range))

        for _number in _range:
            _data = table.new_data()
            _data.testid = _number
            _data.use_test = True
            _testname = "Test{0:s}".format(str(_number).rjust(_width, "0"))
            _data.testname = _testname
            _data.path = "/blo/bka/{0:s}".format(_testname)
            table.add(_data)
        return

    def test_bulk_insert_01(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _table.name = "tester01"

        self._fill_bulk(_table, 500)

        count = _sqlite.insert(_table.name, _table.names, _table.data)
        self.assertEqual(count, 500)

        self._clean(_sqlite)
        return

    def test_bulk_insert_02(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _table.name = "tester01"

        self._fill_bulk(_table, 1000)

        count = _sqlite.insert(_table.name, _table.names, _table.data)
        self.assertEqual(count, 1000)

        self._clean(_sqlite)
        return

    def test_bulk_insert_03(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _table.name = "tester01"

        self._fill_bulk(_table, 5000)

        count = _sqlite.insert(_table.name, _table.names, _table.data)
        self.assertEqual(count, 5000)

        self._clean(_sqlite)
        return

    def test_bulk_insert_04(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _table.name = "tester01"

        self._fill_bulk(_table, 10000)

        count = _sqlite.insert(_table.name, _table.names, _table.data)
        self.assertEqual(count, 10000)

        self._clean(_sqlite)
        return

    def test_bulk_insert_05(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _table.name = "tester01"

        self._fill_bulk(_table, 20000)

        count = _sqlite.insert(_table.name, _table.names, _table.data)
        self.assertEqual(count, 20000)

        self._clean(_sqlite)
        return

    def test_bulk_insert_06(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _table.name = "tester01"

        self._fill_bulk(_table, 50000)

        count = _sqlite.insert(_table.name, _table.names, _table.data)
        self.assertEqual(count, 50000)

        self._clean(_sqlite)
        return

    def test_bulk_insert_07(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _table.name = "tester01"

        self._fill_bulk(_table, 100000)

        count = _sqlite.insert(_table.name, _table.names, _table.data)
        self.assertEqual(count, 100000)

        self._clean(_sqlite)
        return

    def test_bulk_insert_08(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _table.name = "tester01"

        self._fill_bulk(_table, 10)

        count = _sqlite.insert(_table.name, _table.names, _table.data)
        self.assertEqual(count, 10)

        self._fill_bulk(_table, 10)

        count = _sqlite.insert(_table.name, _table.names, _table.data)
        self.assertEqual(count, 0)

        self._clean(_sqlite)
        return

    def test_update_01(self):
        _sqlite = copy_sqlite(filename="test_update.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        _new = get_data_07()
        sql_filter = "testid = ?"

        _check = _sqlite.update(_table.name, _table.names, _new, sql_filter, 4)
        self.assertTrue(_check)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.connect', new=get_sqlite_return_false())
    def test_update_02(self):
        _sqlite = copy_sqlite(filename="test_update.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        _new = get_data_07()
        sql_filter = "testid = ?"

        _check = _sqlite.update(_table.name, _table.names, _new, sql_filter, 4)
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.release', new=get_sqlite_return_false())
    def test_update_03(self):
        _sqlite = copy_sqlite(filename="test_update.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        _new = get_data_07()
        sql_filter = "testid = ?"

        _check = _sqlite.update(_table.name, _table.names, _new, sql_filter, 4)
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.commit', new=get_sqlite_return_false())
    def test_update_04(self):
        _sqlite = copy_sqlite(filename="test_update.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        _new = get_data_07()
        sql_filter = "testid = ?"

        _check = _sqlite.update(_table.name, _table.names, _new, sql_filter, 4)
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_operational_error())
    def test_update_05(self):
        _sqlite = copy_sqlite(filename="test_update.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        _new = get_data_07()
        sql_filter = "testid = ?"

        _check = _sqlite.update(_table.name, _table.names, _new, sql_filter, 4)
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_integrity_error())
    def test_update_06(self):
        _sqlite = copy_sqlite(filename="test_update.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        _new = get_data_07()
        sql_filter = "testid = ?"

        _check = _sqlite.update(_table.name, _table.names, _new, sql_filter, 4)
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    def test_update_07(self):
        _sqlite = copy_sqlite(filename="test_update.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        _new = get_data_08()
        sql_filter = "testid = ?"

        _check = _sqlite.update(_table.name, _table.names, _new, sql_filter, 4)
        self.assertFalse(_check)

        self._clean(_sqlite)
        return

    def test_select_01(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        _data_list = _sqlite.select(table_name=_table.name, names=[], sql_filter="", data=[])
        _count = len(_data_list)

        _data = (1, True, "Test01", "testers/")
        _check_data = _data_list[0]

        self.assertEqual(_count, 6)
        self.assertSequenceEqual(_data, _check_data)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.connect', new=get_sqlite_return_false())
    def test_select_02(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        _data_list = _sqlite.select(table_name=_table.name, names=[], sql_filter="", data=[])
        self.assertIsNone(_data_list)
        return

    @mock.patch('bbutil.database.sqlite.manager.Connection.release', new=get_sqlite_return_false())
    def test_select_03(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        _data_list = _sqlite.select(table_name=_table.name, names=[], sql_filter="", data=[])
        self.assertIsNone(_data_list)
        return

    def test_select_04(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        sql_filter = "testid = ?"
        data_filter = [1]

        _data_list = _sqlite.select(table_name=_table.name, names=[], sql_filter=sql_filter, data=data_filter)
        _count2 = len(_data_list)

        _data = (1, True, "Test01", "testers/")
        _check_data = _data_list[0]

        self.assertEqual(_count2, 1)
        self.assertSequenceEqual(_data, _check_data)
        return

    def test_select_05(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        sql_filter = "testid = ?"
        data_filter = [1]
        names = [
            "testid",
            "use_test"
        ]

        _data_list = _sqlite.select(table_name=_table.name, names=names, sql_filter=sql_filter, data=data_filter)
        _count2 = len(_data_list)

        _data = (1, True)
        _check_data = _data_list[0]

        self.assertEqual(_count2, 1)
        self.assertSequenceEqual(_data, _check_data)
        return

    @mock.patch('sqlite3.connect', new=get_sqlite_operational_error())
    def test_select_06(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        sql_filter = "testid = ?"
        data_filter = [1]
        names = [
            "testid",
            "use_test"
        ]

        _data_list = _sqlite.select(table_name=_table.name, names=names, sql_filter=sql_filter, data=data_filter)
        self.assertIsNone(_data_list)
        return

    def test_select_07(self):
        _sqlite = get_sqlite(filename="test_select.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)

        sql_filter = "testid = ?"
        data_filter = [15235670141346654134]
        names = [
            "testid",
            "use_test"
        ]

        _data_list = _sqlite.select(table_name=_table.name, names=names, sql_filter=sql_filter, data=data_filter)

        self.assertIsNone(_data_list)
        return

    def _compare_bulk(self, data):
        _width = len(str(500))

        _number = 0
        for item in data:
            item_testid = item[0]
            item_use_test = item[1]
            item_testname = item[2]
            item_path = item[3]

            self.assertEqual(item_testid, _number)
            self.assertTrue(item_use_test)

            _testname = "Test{0:s}".format(str(_number).rjust(_width, "0"))
            _path = "/blo/bka/{0:s}".format(_testname)
            self.assertEqual(item_testname, _testname)
            self.assertEqual(item_path, _path)

            _number += 1
        return

    def test_bulk_insert_select_01(self):
        _sqlite = copy_sqlite(filename="test_check_table.sqlite", path="testdata/database")
        _sqlite.prepare()

        _table = get_table_01(_sqlite)
        _table.name = "tester01"

        self._fill_bulk(_table, 500)

        count = _sqlite.insert(_table.name, _table.names, _table.data)
        self.assertEqual(count, 500)

        data = _sqlite.select(_table.name, _table.names, "", [])
        count = len(data)
        self.assertEqual(count, 500)

        self._compare_bulk(data)

        self._clean(_sqlite)
        return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

from typing import Optional
from unittest.mock import Mock

__all__ = [
    "CatchBacks",
    "MockPopen1",
    "MockPopen2",
    "MockPopen3"
]


class CatchBacks(object):

    def __init__(self):
        self.stdout = []
        self.stderr = []
        return

    def add_stdout(self, data):
        if data is None:
            return
        self.stdout.append(data)
        return

    def add_stderr(self, data):
        if data is None:
            return
        self.stderr.append(data)
        return


def get_stdout() -> list:

    _line1 = "TEST"

    _excec = UnicodeDecodeError('funnycodec', _line1.encode(), 1, 2, 'This is just a fake reason!')

    _line2 = Mock()
    _line2.decode = Mock(side_effect=_excec)

    _stdout = [
        _line1.encode(),
        _line2,
        None
    ]
    return _stdout


def get_stderr() -> list:
    _line1 = "ERROR!"

    _stderr = [
        _line1.encode(),
        None
    ]

    return _stderr


class MockPopen1(object):

    def __init__(self, commands, stdout, stderr, stdin=None):

        self.test_stdout = stdout
        self.test_stderr = stderr
        self.test_stdin = stdin
        self._poll = 0
        self.stdout = get_stdout()
        self.stderr = []
        self.returncode = 0
        return

    def poll(self) -> Optional[int]:
        if self._poll == 10:
            self._poll = 0
            return 1
        self._poll += 1
        return None


class MockPopen2(object):

    def __init__(self, commands, stdout, stderr, stdin=None):

        self.test_stdout = stdout
        self.test_stderr = stderr
        self.test_stdin = stdin
        self._poll = 0
        self.stdout = get_stdout()
        self.stderr = get_stderr()
        self.returncode = 1
        return

    def poll(self) -> Optional[int]:
        if self._poll == 10:
            self._poll = 0
            return 1
        self._poll += 1
        return None


class MockPopen3(object):

    def __init__(self, commands, stdout, stderr, stdin=None):

        self.test_stdout = stdout
        self.test_stderr = stderr
        self.test_stdin = stdin
        self._poll = 0
        self.stdout = get_stdout()
        self.stderr = []
        self.returncode = 1
        return

    def poll(self) -> Optional[int]:
        if self._poll == 10:
            self._poll = 0
            return 1
        self._poll += 1
        return None



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import sys
import os

from dataclasses import dataclass

from bbutil.utils import full_path
from bbutil.logging import Logging
from bbutil.app import Console, Config

from testdata.app.config import AppConfig

__all__ = [
    "AppConsole"
]

_index = {
    0: ["INFORM", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"],
    1: ["INFORM", "DEBUG1", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"],
    2: ["INFORM", "DEBUG1", "DEBUG2", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"],
    3: ["INFORM", "DEBUG1", "DEBUG2", "DEBUG3", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"]
}

if sys.platform == "win32":
    _filename1 = full_path("{0:s}/testdata/config01_win.json".format(os.getcwd()))
else:
    _filename1 = full_path("{0:s}/testdata/config01.json".format(os.getcwd()))


@dataclass
class AppConsole(Console):

    filename: str = ""
    return_start: bool = True
    return_stop: bool = True

    def create_logging(self) -> Logging:
        _log = Logging()
        _log.setup(app="Test", level=2, index=_index)

        console = _log.get_writer("console")
        _log.register(console)
        _log.open()
        return _log

    def create_config(self) -> Config:
        _work = "{0:s}/test".format(os.getcwd())
        if os.path.exists(_work) is False:
            os.mkdir(_work)

        _config = AppConfig(use_config=True, config_filename=self.filename)
        return _config

    def init(self):
        self.filename = _filename1
        return

    def start(self) -> bool:
        return self.return_start

    def stop(self) -> bool:
        return self.return_stop



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2023, Kai Raphahn <kai.raphahn@laburec.de>
#

from dataclasses import dataclass, field
from typing import Optional, List, Dict, Any, Tuple
from enum import Enum

import bbutil

from bbutil.database import Column, Types, Data, DataType, select_interval
from bbutil.database.sqlite import SQLite


__all__ = [
    "Table"
]


class _InitType(Enum):

    has_table = 0
    has_old_table = 1
    has_no_table = 2
    has_no_columns = 3


@dataclass
class Table(object):

    name: str = ""
    old_name: str = ""
    _counter: int = 0
    keyword: str = ""
    sqlite: Optional[SQLite] = None
    data: List[Data] = field(default_factory=list)
    index: Dict[Any, List[Data]] = field(default_factory=dict)
    columns: List[Column] = field(default_factory=list)
    missing_columns: List[Column] = field(default_factory=list)
    invalid_columns: List[Column] = field(default_factory=list)
    drop_columns: List[str] = field(default_factory=list)
    names: List[str] = field(default_factory=list)
    suppress_warnings: bool = False

    def clear(self):
        self.data.clear()
        self.index.clear()
        return

    @property
    def data_count(self) -> int:
        _counter = len(self.data)
        return _counter

    def check(self) -> int:
        self._counter = self.sqlite.count(self.name)
        return self._counter

    @property
    def column_list(self) -> list:
        _columns = []
        for _col in self.columns:
            _columns.append(_col.create)
        return _columns

    @property
    def unique_list(self) -> list:
        _unique = []
        for _col in self.columns:
            if _col.unique is False:
                continue
            _unique.append(_col.name)
        return _unique

    def new_data(self) -> Data:
        key_list = []
        value_list = []

        for _column in self.columns:
            # noinspection PyTypeChecker
            _datatype: DataType = _column.type.value

            key_list.append(_column.name)
            value_list.append(_datatype.value)

        _data = Data(keys=key_list, values=value_list)
        return _data

    def add_column(self, name: str, data_type: Types, unique: bool = False, primarykey: bool = False,
                   keyword: bool = False):
        for _column in self.columns:
            if _column.name == name:
                return

        _column = Column(name=name, primarykey=primarykey, type=data_type, unique=unique)
        self.columns.append(_column)

        if keyword is True:
            self.keyword = name

        if primarykey is False:
            self.names.append(name)
        return

    def get_column(self, name: str) -> Optional[Column]:
        for _column in self.columns:
            if _column.name == name:
                return _column
        return None

    def _process_datalist(self, data_list: List[Tuple], verbose: bool = True) -> Optional[List[Data]]:
        if data_list is None:
            if (self.suppress_warnings is False) and (verbose is True):
                bbutil.log.warn(self.name, "No data!")
            return None

        _count = len(data_list)
        if _count == 0:
            if (self.suppress_warnings is False) and (verbose is True):
                bbutil.log.warn(self.name, "No data!")
            return None

        progress = None
        if verbose is True:
            bbutil.log.inform("Table", "Load {0:d} from {1:s}".format(_count, self.name))
            progress = bbutil.log.progress(_count, select_interval(_count))

        _result = []
        _count = 0
        for _data in data_list:
            _number = 0

            key_list = []
            value_list = []

            for _col in self.columns:
                try:
                    _value = _data[_number]
                except IndexError as e:
                    bbutil.log.error("Problem with data item {0:d}!".format(_count))
                    bbutil.log.error("Column {0:d} ({1:s}) not found!".format(_number, _col.name))
                    bbutil.log.exception(e)
                    return None

                key_list.append(_col.name)

                if _col.type is Types.bool:
                    if _value == 1:
                        _value = True
                    else:
                        _value = False

                value_list.append(_value)
                _number += 1

            _count += 1

            _entry = Data(keys=key_list, values=value_list)
            _result.append(_entry)

            if progress is not None:
                progress.inc()

        if verbose is True:
            bbutil.log.clear()
        return _result

    def select(self, sql_filter: str = "", names=None, data_values=None, verbose: bool = True) -> List[Data]:
        if names is None:
            names = []

        if data_values is None:
            data_values = []

        _data_list = self.sqlite.select(table_name=self.name, sql_filter=sql_filter, names=names, data=data_values)
        _result = self._process_datalist(_data_list, verbose)

        if _result is None:
            return []

        return _result

    def store(self, data: Data = None) -> int:
        _data = data
        if data is None:
            _data = self.data

        _count = self.sqlite.insert(self.name, self.names, _data)
        return _count

    def update(self, data: Data, data_filter: str, filter_value=None) -> bool:
        _check = self.sqlite.update(self.name, self.names, data, data_filter, filter_value)
        return _check

    def _check_table(self) -> _InitType:
        _type = _InitType.has_no_table

        if len(self.columns) == 0:
            _type = _InitType.has_no_columns
            return _type

        _check = self.sqlite.check_table(self.name)
        if _check is True:
            _type = _InitType.has_table
        else:

            if self.old_name != "":
                _check = self.sqlite.check_table(self.old_name)
                if _check is True:
                    _type = _InitType.has_old_table

        return _type

    def _create_table(self) -> bool:
        _columns = []
        for _col in self.columns:
            _columns.append(_col.create)

        _unique = []
        for _col in self.columns:
            if _col.unique is False:
                continue
            _unique.append(_col.name)

        _check = self.sqlite.create_table(self.name, _columns, _unique)
        if _check is False:
            return False
        return True

    def _rename_table(self) -> bool:
        _check = self.sqlite.rename_table(self.old_name, self.name)
        if _check is False:
            return False
        return True

    def upgrade(self) -> bool:
        check = self.check_scheme()
        if check is True:
            return True

        count_invalid = len(self.invalid_columns)

        if count_invalid > 0:
            bbutil.log.error("Unable to change columns!")
            return False

        _columns = []
        for column in self.missing_columns:
            if column.primarykey is True:
                bbutil.log.error("Unable to add new primarykey column: {0:s}".format(column.name))
                return False

            if column.unique is True:
                bbutil.log.error("Unable to add new unique column: {0:s}".format(column.name))
                return False
            _columns.append(column.create)

        check = self.sqlite.add_columns(self.name, _columns)
        if check is False:
            return False

        if len(self.drop_columns) == 0:
            return True

        check = self.sqlite.check_minmal_version(3, 35, 0)
        if check is False:
            return True

        check = self.sqlite.drop_columns(self.name, self.drop_columns)
        if check is False:
            return False

        return True

    def init(self) -> bool:
        if bbutil.log is None:
            return False

        self.sqlite.prepare()

        _type = self._check_table()

        _check = False

        if _type is _InitType.has_no_columns:
            _check = False

        if _type is _InitType.has_no_table:
            _check = self._create_table()

        if _type is _InitType.has_old_table:
            _check = self._rename_table()

        if _type is _InitType.has_table:
            _check = True

        if _check is False:
            return False

        _count = self.sqlite.count(self.name)
        if _count == -1:
            return False

        self._counter = _count
        return True

    def add(self, item: Data):
        self.data.append(item)

        if self.keyword == "":
            return

        _keyword = getattr(item, self.keyword, None)
        if _keyword is None:
            raise Exception("Keyword is missing!")

        try:
            _list = self.index[_keyword]
        except KeyError:
            self.index[_keyword] = []
            _list = self.index[_keyword]

        _list.append(item)
        return

    def check_scheme(self) -> bool:
        if bbutil.log is None:
            return False

        if len(self.columns) == 0:
            bbutil.log.error("No columns: {0:s}".format(self.name))
            return False

        _schemes = {}
        _columns = {}

        _data = self.sqlite.get_scheme(self.name)
        if _data is None:
            bbutil.log.error("Scheme for {0:s} not found!".format(self.name))
            return False

        self.missing_columns.clear()
        self.invalid_columns.clear()
        self.drop_columns.clear()

        for item in _data:
            _schemes[item[0]] = item[1]

        for _col in self.columns:
            _columns[_col.name] = _col

        for _name in _schemes:
            try:
                _col = _columns[_name]
            except KeyError:
                bbutil.log.error("Column {0:s} found, but no definition exists!".format(_name))
                self.drop_columns.append(_name)

        for _column in self.columns:
            try:
                _value = _schemes[_column.name]
            except KeyError:
                bbutil.log.error("Column {0:s} in {1:s} not found!".format(_column.name, self.name))
                self.missing_columns.append(_column)
                return False

            expected_value = _column.type.value.type

            if expected_value != _value:
                _error = "Column {0:s} in {1:s} does not match: found {2:s}, expected {3:s}".format(_column.name,
                                                                                                    self.name,
                                                                                                    _value,
                                                                                                    expected_value)
                bbutil.log.error(_error)
                self.invalid_columns.append(_column)
                return False

        if len(self.drop_columns) > 0:
            return False

        return True

    def load(self) -> int:
        bbutil.log.inform(self.name, "Load {0:s}...".format(self.name))

        _items = self.select()
        _count = len(_items)

        _max = _count + 1
        _progress = bbutil.log.progress(_max, select_interval(_max))

        for _item in _items:
            self.add(_item)
            _progress.inc()

        bbutil.log.clear()

        _count = self.data_count
        return _count



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import unittest

from unittest.mock import patch, Mock

import bbutil
from tests.helper.console import AppConsole
from tests.helper.module import config_modules, config_modules_2


__all__ = [
    "TestConsole"
]

oserror = OSError("Something strange did happen!")
mock_oserror = Mock(side_effect=oserror)
mock_remove = Mock()


class TestConsole(unittest.TestCase):
    """Testing class for locking module."""

    def setUp(self):
        return

    def test_setup_01(self):
        _console = AppConsole(module_config=config_modules)

        _check = _console.setup()
        self.assertTrue(_check)
        return

    def test_setup_02(self):
        _console = AppConsole(module_config=[])
        _console.module_path = "testdata.app.xcommands"

        _check = _console.setup()
        self.assertFalse(_check)
        return

    def test_setup_03(self):
        _console = AppConsole(module_config=config_modules)
        _console.filename = "fuhhhhhhh"

        _check = _console.setup()
        self.assertFalse(_check)
        return

    def test_execute_01(self):
        _console = AppConsole(module_config=config_modules)

        _argv = [
            "run-tests.py",
            "test01"
        ]

        _check1 = _console.setup()

        with patch("sys.argv", _argv):
            _ret = _console.execute()

        self.assertTrue(_check1)
        self.assertEqual(_ret, 0)
        return

    def test_execute_02(self):
        _console = AppConsole(module_config=config_modules)

        _check1 = _console.setup()

        # noinspection PyTypeChecker
        bbutil.set_config(None)

        _ret = _console.execute()

        self.assertTrue(_check1)
        self.assertEqual(_ret, -1)
        return

    def test_execute_03(self):
        _console = AppConsole(module_config=config_modules)

        _argv = [
            "run-tests.py",
            "xxxxtest01"
        ]

        _check1 = _console.setup()

        with patch("sys.argv", _argv):
            _ret = _console.execute()

        self.assertTrue(_check1)
        self.assertEqual(_ret, -1)
        return

    def test_execute_04(self):
        _console = AppConsole(module_config=config_modules_2)

        _argv = [
            "run-tests.py",
            "test01"
        ]

        _check1 = _console.setup()

        with patch("sys.argv", _argv):
            _ret = _console.execute()

        self.assertTrue(_check1)
        self.assertEqual(_ret, 2)
        return

    def test_execute_05(self):
        _console = AppConsole(module_config=config_modules)

        _argv = [
            "run-tests.py",
            "test01"
        ]

        _check1 = _console.setup()
        _console.return_start = False

        with patch("sys.argv", _argv):
            _ret = _console.execute()

        self.assertTrue(_check1)
        self.assertEqual(_ret, 1)
        return

    def test_execute_06(self):
        _console = AppConsole(module_config=config_modules)

        _argv = [
            "run-tests.py",
            "test01"
        ]

        _check1 = _console.setup()
        _console.return_stop = False

        with patch("sys.argv", _argv):
            _ret = _console.execute()

        self.assertTrue(_check1)
        self.assertEqual(_ret, 4)
        return

    def test_execute_07(self):
        _console = AppConsole(module_config=config_modules)

        _argv = [
            "run-tests.py",
            "test03"
        ]

        _check1 = _console.setup()

        with patch("sys.argv", _argv):
            _ret = _console.execute()

        self.assertTrue(_check1)
        self.assertEqual(_ret, 3)
        return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import sys
import colorama
import platform

from typing import Dict, TextIO, List

from bbutil.logging.types import Writer, Message
from bbutil.utils import get_terminal_size

__all__ = [
    "ConsoleWriter"
]

colorama.init()

classname = "ConsoleWriter"
RESET_ALL = colorama.Style.RESET_ALL


class _Style(object):

    def __init__(self, name: str, text: str, foreground: str, background: str):
        self.name = name
        self.text = text
        self.foreground = foreground
        self.background = background
        return

    @property
    def scheme(self) -> str:
        scheme = ""

        scheme += self._get_colorama(colorama.Style, self.text)
        scheme += self._get_colorama(colorama.Fore, self.foreground)
        scheme += self._get_colorama(colorama.Back, self.background)
        return scheme

    @staticmethod
    def _get_colorama(attribute, name):
        try:
            ret = getattr(attribute, name)
        except AttributeError:
            ret = ""

        return ret


_index = ["INFORM", "DEBUG1", "DEBUG2", "DEBUG3", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"]

_schemes = {
    "INFORM": _Style("INFORM", "BRIGHT", "GREEN", ""),
    "DEBUG1": _Style("DEBUG1", "", "WHITE", "BLACK"),
    "DEBUG2": _Style("DEBUG2", "DIM", "CYAN", "BLACK"),
    "DEBUG3": _Style("DEBUG3", "BRIGHT", "BLACK", "BLACK"),
    "WARN": _Style("INFORM", "BRIGHT", "MAGENTA", ""),
    "ERROR": _Style("ERROR", "BRIGHT", "RED", ""),
    "EXCEPTION": _Style("EXCEPTION", "BRIGHT", "RED", ""),
    "TIMER": _Style("TIMER", "BRIGHT", "YELLOW", "")
}


class ConsoleWriter(Writer):

    def __init__(self):
        Writer.__init__(self, "Console", _index)

        self.styles: Dict[str, _Style] = _schemes
        self.encoding: str = ""
        self.app_space: int = 0
        self.text_space: int = 15
        self.seperator: str = "|"
        self.length: int = 0
        self.error_index: List[str] = ["ERROR", "EXCEPTION"]
        self.use_error = False
        self.stdout: TextIO = sys.stdout
        self.stderr: TextIO = sys.stderr

        size_x, size_y = get_terminal_size()

        self.line_width: int = size_x
        self.bar_len: int = 50
        return

    def setup(self, **kwargs):
        item = kwargs.get("text_space", None)
        if item is not None:
            self.text_space = item

        item = kwargs.get("app_space", None)
        if item is not None:
            self.app_space = item

        item = kwargs.get("seperator", None)
        if item is not None:
            self.seperator = item

        item = kwargs.get("error_index", None)
        if item is not None:
            self.error_index = item

        item = kwargs.get("bar_len", None)
        if item is not None:
            self.bar_len = item
        return

    def add_style(self, name: str, text: str, foreground: str, background: str):
        style = _Style(name, text, foreground, background)
        self.styles[style.name] = style
        return

    def open(self) -> bool:
        self.encoding = self.stdout.encoding
        os_name = platform.system()
        if self.encoding is None:  # pragma: no cover
            if os_name == "Windows":
                self.encoding = "cp850"

        return True

    def close(self) -> bool:  # pragma: no cover
        return True

    def clear(self) -> bool:
        filler = ' ' * self.length
        content = "\r{0:s}\r".format(filler)

        if self.use_error is True:
            self.stderr.write(content)
        else:
            self.stdout.write(content)
        return True

    def write(self, item: Message):
        error = False

        if item.level in self.error_index:
            error = True

        if item.level == "PROGRESS":
            self._write_progress(item)
            return

        if item.raw is True:
            self._write(error, item.content, "Unable to log content due de- or encoding error!")
        else:
            content = self._create_color(item, item.content)
            fallback = self._create_color(item, "Unable to log content due de- or encoding error!")
            self._write(error, content, fallback)
        return

    def _write_progress(self, item: Message):
        self.use_error = False
        value = self.bar_len * item.progress.counter / float(item.progress.limit)

        filled_len = int(round(value))

        bars = '=' * filled_len
        filler = '-' * (self.bar_len - filled_len)

        percents = "{0:.1f}%".format(round(item.progress.value, 1)).rjust(6, ' ')

        limit = 3 + (item.progress.length * 2)
        counter = "({0:d}/{1:d})".format(item.progress.counter, item.progress.limit).rjust(limit)

        content = " [{0:s}{1:s}] {2:s} {3:s}".format(bars, filler, percents, counter)

        if len(content) > self.line_width:
            return

        self.length = len(content) + 1

        try:
            output = content.encode(self.encoding, "replace").decode(self.encoding, "replace")
        except UnicodeDecodeError:  # pragma: no cover
            return

        line = '\r' + output
        self.stdout.write(line)
        return

    def _create_color(self, item: Message, text: str) -> str:
        _app_space = self.app_space
        if self.app_space == 0:
            _app_space = len(item.app) + 5
        appname = "{0:s} ".format(item.app).ljust(_app_space)
        scheme = self.styles[item.level].scheme

        if item.tag == "":
            content = "{0:s}{1:s}{2:s} {3:s}{4:s}".format(RESET_ALL, appname, scheme, text, RESET_ALL)
        else:
            tag = item.tag.ljust(self.text_space)
            content = "{0:s}{1:s}{2:s} {3:s}{4:s} {5:s}{6:s}".format(RESET_ALL, appname, scheme, tag, self.seperator,
                                                                     RESET_ALL, text)
        return content

    def _write(self, error: bool, content: str, fallback: str):
        output = ""

        try:
            output += content.encode(self.encoding, "replace").decode(self.encoding, "replace")
            self.data = output

        except UnicodeDecodeError as e:  # pragma: no cover
            output += "UnicodeDecodeError: " + str(e) + "\n"
            output += fallback + "\n"

        except UnicodeEncodeError as e:  # pragma: no cover
            output += "UnicodeEncodeError: " + str(e) + "\n"
            output += fallback + "\n"

        # Now we write the output to the console
        output += '\n'

        self.length = len(output)

        if error is True:
            self.use_error = True
            self.stderr.write(output)
        else:
            self.use_error = False
            self.stdout.write(output)
        return



#!/usr/bin/python3
# coding=utf-8

# Copyright (C) 2020, Siemens Healthcare Diagnostics Products GmbH
# Licensed under the Siemens Inner Source License 1.2, see LICENSE.md.

import abc
import sys

from dataclasses import dataclass, field
from typing import Optional, List
from abc import ABCMeta

import bbutil

from bbutil.logging import Logging
from bbutil.app.manager import ModuleManager
from bbutil.app.config import Config
from bbutil.app.module import Module

__all__ = [
    "Console"
]


@dataclass
class Console(metaclass=ABCMeta):

    command_id: str = ""
    module: Optional[Module] = None
    module_config: List[dict] = field(default_factory=list)

    def __post_init__(self):
        self.init()
        return

    def _set_command(self) -> bool:
        if bbutil.config is None:
            bbutil.log.error("Config is missing!")
            return False

        if self.command_id == "":
            command_names = sys.argv[1:]

            for _command in command_names:
                check = bbutil.module.has_command(_command)
                if check is False:
                    continue

                self.command_id = _command
                break

        if self.command_id == "":
            bbutil.config.prepare_parser()
            bbutil.config.parser.print_help()
            return False

        self.module = bbutil.module.get_command(self.command_id)
        bbutil.log.debug1("Console", self.command_id)
        return True

    @abc.abstractmethod
    def create_logging(self) -> Logging:
        pass

    @abc.abstractmethod
    def create_config(self) -> Config:
        pass

    @abc.abstractmethod
    def init(self):
        pass

    @abc.abstractmethod
    def start(self) -> bool:
        pass

    @abc.abstractmethod
    def stop(self) -> bool:
        pass

    def _setup_module(self) -> bool:
        if len(self.module_config) == 0:
            bbutil.log.error("No modules!")
            return False

        _modules = ModuleManager()

        _modules.init(self.module_config)

        bbutil.set_module(_modules)
        return True

    def _setup_config(self) -> bool:
        _config = self.create_config()

        check = _config.init()
        if check is False:
            return False

        bbutil.set_config(_config)
        return True

    def setup(self) -> bool:
        _log = self.create_logging()
        bbutil.set_log(_log)

        _version = sys.version.replace('\n', '- ')
        bbutil.log.debug1("Python", _version)

        _check = self._setup_module()
        if _check is False:
            return False

        _check = self._setup_config()
        if _check is False:
            return False

        bbutil.log.setup(level=bbutil.config.verbose)
        return True

    def execute(self) -> int:
        check = self._set_command()
        if check is False:
            return -1

        bbutil.log.inform("App", "Start {0:s}".format(self.command_id))

        check = self.start()
        if check is False:
            return 1

        check = self.module.load()
        if check is False:
            return 2

        for _worker in self.module.workers:
            check = _worker.execute()
            if check is True:
                continue

            return 3

        check = self.stop()
        if check is False:
            return 4

        bbutil.log.close()
        return 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import coverage

cov = coverage.Coverage()
cov.start()

from bbutil.logging import Logging
from bbutil.utils import full_path

from typing import List
from optparse import OptionParser

from bbutil.utils import check_dict, openjson

import os
import sys
import unittest
import warnings
import time

from unittest.signals import registerResult
import unittest.result as result


log: Logging = Logging()


class Module(object):

    def __init__(self):
        self.id: str = ""
        self.path: str = ""
        self.classname: str = ""
        self.tests: List[str] = []
        return

    def load(self, data: dict) -> bool:

        check = check_dict(data, ["id", "path", "classname", "tests"])
        if check is False:
            log.error("Module config not complete!")
            return False

        self.id = data["id"]
        self.path = data["path"]
        self.classname = data["classname"]
        self.tests = data["tests"]
        return True


class _WritelnDecorator(object):
    """Used to decorate file-like objects with a handy 'writeln' method"""
    def __init__(self, stream):
        self.stream = stream

    def __getattr__(self, attr):
        if attr in ('stream', '__getstate__'):
            raise AttributeError(attr)
        return getattr(self.stream, attr)

    def writeln(self, arg=None):
        if arg:
            self.write(arg)
        self.write('\n')  # text-mode streams translate to \r\n if needed


class TextTestResult(result.TestResult):
    """A test result class that can print formatted text results to a stream.

    Used by TextTestRunner.
    """
    separator1 = '=' * 70
    separator2 = '-' * 70

    def __init__(self, stream, descriptions, verbosity):
        super(TextTestResult, self).__init__(stream, descriptions, verbosity)
        self.stream = stream
        self.showAll = verbosity > 1
        self.dots = verbosity == 1
        self.descriptions = descriptions
        self.current_test = ""
        return

    @staticmethod
    def _get_description(test):
        desc = str(test)
        # doc_first_line = test.shortDescription()
        # if self.descriptions and doc_first_line:
        #     return '\n'.join((str(test), doc_first_line))
        # else:
        #     return str(test)
        return desc

    def startTest(self, test):
        super(TextTestResult, self).startTest(test)
        self.current_test = self._get_description(test)
        return

    def addSuccess(self, test):
        super(TextTestResult, self).addSuccess(test)
        log.inform("OK", self.current_test)
        return

    def addError(self, test, err):
        super(TextTestResult, self).addError(test, err)
        log.warn("ERROR", self.current_test)
        return

    def addFailure(self, test, err):
        super(TextTestResult, self).addFailure(test, err)
        log.warn("FAIL", self.current_test)
        return

    def addSkip(self, test, reason):
        super(TextTestResult, self).addSkip(test, reason)
        content = "skipped {0!r}".format(reason)
        log.warn(self.current_test, content)
        return

    def addExpectedFailure(self, test, err):
        super(TextTestResult, self).addExpectedFailure(test, err)
        log.warn(self.current_test, "expected failure")
        return

    def addUnexpectedSuccess(self, test):
        super(TextTestResult, self).addUnexpectedSuccess(test)
        log.warn(self.current_test, "unexpected success")
        return

    # noinspection PyPep8Naming
    def printErrors(self):
        self._print_error_list('ERROR', self.errors)
        self._print_error_list('FAIL', self.failures)
        return

    def _print_error_list(self, flavour, errors):
        for test, err in errors:
            log.error(flavour)
            log.raw(self.separator1 + "\n")
            log.raw(self._get_description(test) + "\n")
            log.raw(self.separator2 + "\n")
            log.raw("%s" % err)
        return


class TextTestRunner(object):
    """A test runner class that displays results in textual form.

    It prints out the names of tests as they are run, errors as they
    occur, and a summary of the results at the end of the test run.
    """
    resultclass = TextTestResult

    def __init__(self, name="", stream=None, descriptions=True, verbosity=1,
                 failfast=False, buffer=False, resultclass=None, warning=None,
                 *, tb_locals=False):
        """Construct a TextTestRunner.

        Subclasses should accept **kwargs to ensure compatibility as the
        interface changes.
        """
        if stream is None:
            stream = sys.stderr
        self.name = name
        self.stream = _WritelnDecorator(stream)
        self.descriptions = descriptions
        self.verbosity = verbosity
        self.failfast = failfast
        self.buffer = buffer
        self.tb_locals = tb_locals
        self.warnings = warning
        if resultclass is not None:
            self.resultclass = resultclass

    def _make_result(self):
        return self.resultclass(self.stream, self.descriptions, self.verbosity)

    def run(self, test):
        """Run the given test case or test suite.

        :param test: test suite.
        :type test: unittest.TestSuite

        :return: test result.
        :rtype: TextTestResult
        """
        test_result = self._make_result()
        registerResult(test_result)
        test_result.failfast = self.failfast
        test_result.buffer = self.buffer
        test_result.tb_locals = self.tb_locals
        with warnings.catch_warnings():
            if self.warnings:
                # if self.warnings is set, use it to filter all the warnings
                warnings.simplefilter(self.warnings)
                # if the filter is 'default' or 'always', special-case the
                # warnings from the deprecated unittest methods to show them
                # no more than once per module, because they can be fairly
                # noisy.  The -Wd and -Wa flags can be used to bypass this
                # only when self.warnings is None.
                if self.warnings in ['default', 'always']:
                    warnings.filterwarnings('module', category=DeprecationWarning,
                                            message='Please use assert instead.')
            start_time = time.time()
            start_test_run = getattr(test_result, 'startTestRun', None)
            if start_test_run is not None:
                start_test_run()
            try:
                test(test_result)
            finally:
                stop_test_run = getattr(test_result, 'stopTestRun', None)
                if stop_test_run is not None:
                    stop_test_run()
            stop_time = time.time()
        time_taken = stop_time - start_time
        test_result.printErrors()
        # if hasattr(result, 'separator2'):
        #     self.stream.writeln(result.separator2)
        run = test_result.testsRun

        logline = "Ran %d test%s in %.3fs" % (run, run != 1 and "s" or "", time_taken)
        log.inform("RUNNER", logline)

        expected_fails = unexpected_successes = skipped = 0
        try:
            results = map(len, (test_result.expectedFailures,
                                test_result.unexpectedSuccesses,
                                test_result.skipped))
        except AttributeError:
            pass
        else:
            expected_fails, unexpected_successes, skipped = results

        infos = []
        if not test_result.wasSuccessful():
            failed, errored = len(test_result.failures), len(test_result.errors)
            if failed:
                log.warn("FAILURES", str(failed))
            if errored:
                log.warn("ERRORS", str(errored))
        else:
            log.inform("RUNNER", "OK")

        if skipped:
            infos.append("skipped=%d" % skipped)

        if expected_fails:
            infos.append("expected failures=%d" % expected_fails)

        if unexpected_successes:
            infos.append("unexpected successes=%d" % unexpected_successes)

        if infos:
            logline = " (%s)" % (", ".join(infos),)
            log.raw(logline)
        return test_result


class TestTask(object):

    def __init__(self):
        """The constructor.
        """
        self.options = None
        self.do_coverage: bool = False
        self.test_count: int = 0

        self.parser: OptionParser = OptionParser("usage: %prog [options]")

        self.parser.add_option("-c", "--config", help="run config file", metavar="PROFILE", type="string",
                               default="tests.json")
        self.parser.add_option("-m", "--module", help="run test module", metavar="MODULE", type="string", default="")
        self.parser.add_option("-t", "--test", help="run test", metavar="TEST", type="string", default="")
        self.parser.add_option("-l", "--list", help="list tests", action="store_true", default=False)

        self._suite = None
        """test suite"""

        self._modules: List[Module] = []
        """stores test config."""
        return

    @property
    def suite(self) -> unittest.TestSuite:
        return self._suite

    @property
    def modules(self) -> List[Module]:
        return self._modules

    def _load_config(self, filename: str) -> bool:
        """Parse config file.

        :param filename: filename of config file.
        :type filename: str

        :returns: True if successfull, otherwise False.
        :rtype: bool
        """

        if os.path.exists(filename) is False:
            log.error("File not found: " + filename)
            return False

        data = openjson(filename)

        check = check_dict(data, ["modules"])

        if check is False:
            log.error("Config is invalid!")
            return False

        for item in data["modules"]:
            module = Module()

            check = module.load(item)
            if check is False:
                continue

            if self.options.module != "":
                if module.id == self.options.module:
                    self._modules.append(module)
            else:
                self._modules.append(module)

        if len(self.modules) == 0:
            log.error("No tests configured!")
            return False

        log.inform("TESTS", "Number of modules: " + str(len(self._modules)))
        return True

    def _print_config(self):

        for module in self._modules:
            log.inform("Module", module.id)

            for test in module.tests:
                log.inform("Test", "\t" + test)
        return

    def prepare(self) -> bool:
        """Start and prepare the test task.

        :returns: True if successfull, otherwise False.
        :rtype: bool
        """

        (options, args) = self.parser.parse_args()

        if options is None:
            log.error("Unable to parse options!")
            return False

        self.options = options

        _filename = os.path.abspath(os.path.normpath(self.options.config))

        check = self._load_config(_filename)
        if check is False:
            return False

        if self.options.list is True:
            self._print_config()
            return True

        self._suite = unittest.TestSuite()
        count = 0

        sys.path.append(os.getcwd())

        for module in self.modules:
            log.inform("RUN", module.id)

            fromlist = [module.classname]
            m = __import__(module.path, globals(), locals(), fromlist)
            c = getattr(m, module.classname)
            for item in module.tests:

                if self.options.test != "":
                    if self.options.test == item:
                        self.suite.addTest(c(item))
                        count += 1
                else:
                    self.suite.addTest(c(item))
                    count += 1

        if count == 0:
            log.error("No tests to run!")
            return False
        self.test_count = count
        log.inform("TESTS", "Tests to run: " + str(count))
        return True

    def run(self) -> bool:
        """Run the test task.

        :returns: True if successfull, otherwise False.
        :rtype: bool
        """
        if self.options.list is True:
            return True

        if self.test_count == 0:
            return True

        runner = TextTestRunner(verbosity=2)

        runner.run(self.suite)
        self.do_coverage = True
        return True


def do_exit(return_value: int):
    log.close()
    sys.exit(return_value)


if __name__ == '__main__':

    log.setup(app="run-tests", level=2)

    console = log.get_writer("console")
    fileio = log.get_writer("file")
    filename = full_path("{0:s}/run-tests.log".format(os.getcwd()))

    console.setup(text_space=15, error_index=["ERROR", "EXCEPTION"])
    fileio.setup(text_space=15, error_index=["ERROR", "EXCEPTION"], filename=filename)
    log.register(console)
    log.register(fileio)

    if log.open() is False:
        do_exit(1)

    main = TestTask()

    if main.prepare() is False:
        do_exit(1)

    if main.run() is False:
        do_exit(1)

    if main.do_coverage is True:
        cov.stop()
        cov.save()
        cov.html_report()

    do_exit(0)



#!/usr/bin/python3
# coding=utf-8
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

from bbutil.database.types import Types, Column, DataType, Data, select_interval
from bbutil.database.sqlite import SQLite
from bbutil.database.table import Table
from bbutil.database.database import Database

__all__ = [
    "sqlite",

    "database",
    "table",
    "types",

    "Types",
    "Column",
    "Data",
    "DataType",
    "select_interval",

    "SQLite",
    "Table",
    "Database"
]



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import io
import os

from datetime import datetime

from bbutil.logging.types import Writer, Message
from bbutil.utils import full_path, print_exception

__all__ = [
    "FileWriter"
]

_index = ["INFORM", "DEBUG1", "DEBUG2", "DEBUG3", "WARN", "ERROR", "EXCEPTION", "TIMER"]

classname = "FileWriter"


class FileWriter(Writer):

    def __init__(self):
        Writer.__init__(self, "File", _index)

        self.filename: str = ""
        self.append_data: bool = False
        self.text_space: int = 15

        # noinspection PyTypeChecker
        self.file: io.FileIO = None
        return

    def setup(self, **kwargs):
        logname = ""
        append_datetime = False
        logpath = ""

        item = kwargs.get("text_space", None)
        if item is not None:
            self.text_space = item

        item = kwargs.get("filename", None)
        if item is not None:
            self.filename = full_path(item)

        item = kwargs.get("logname", None)
        if item is not None:
            logname = item

        item = kwargs.get("logpath", None)
        if item is not None:
            logpath = item

        item = kwargs.get("append_datetime", None)
        if item is not None:
            append_datetime = item

        item = kwargs.get("append_data", None)
        if item is not None:
            self.append_data = item

        if (append_datetime is True) and ((logname == "") or (logpath == "")):
            raise ValueError("For appending datetime these values have to been set: append_datetime, logname, logpath")

        if (append_datetime is True) and (logname != "") and (logpath != ""):
            timestamp = datetime.now().strftime("%Y-%m-%d_%H.%M.%S")
            path = "{0:s}/{1:s}_{2:s}.log".format(logpath, logname, timestamp)
            self.filename = os.path.normpath(path)
        return

    def clear(self) -> bool:
        return True

    def open(self) -> bool:

        if self.filename == "":
            raise ValueError("Filename is missing!")

        if self.append_data is True:
            mode = 'ab'
        else:
            mode = 'wb'

        try:
            self.file = open(self.filename, mode)
        except ValueError as e:
            line = "Filename: {0:s}\nMode: {1:s}".format(self.filename, mode)
            print(line)
            print_exception(e)
            return False
        except OSError as e:
            print_exception(e)
            return False

        return True

    def close(self) -> bool:
        if self.file is not None:
            self.file.close()
            self.file = None
        return True

    def _write_raw(self, item: Message):
        if self.file is None:
            raise ValueError("File is not open!")

        data = item.content.encode('latin-1', 'ignore')

        try:
            self.file.write(data)
            self.file.flush()
        except OSError as e:
            print_exception(e)
            return

        return

    def _write_item(self, item: Message):
        if self.file is None:
            raise ValueError("File is not open!")

        timestamp = item.time.strftime('%Y-%m-%d %H:%M:%S ')
        appname = "{0:s}".format(item.app).ljust(self.text_space)

        if item.tag == "":
            line = "{0:s}{1:s} {2:s}: {3:s}\n".format(timestamp, appname, item.level, item.content)
        else:
            line = "{0:s}{1:s} {2:s}: {3:s} - {4:s}\n".format(timestamp, appname, item.level, item.tag, item.content)

        data = line.encode('latin-1', 'ignore')

        try:
            self.file.write(data)
            self.file.flush()
        except OSError as e:
            print_exception(e)
        return

    def write(self, item: Message):
        if item.raw is True:
            self._write_raw(item)
        else:
            self._write_item(item)
        return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import os
import os.path

from typing import Dict, Optional, Any

from bbutil.lang.domain import Domain

__all__ = [
    "parser",

    "domain",

    "Lang"
]

lang_domain = None


class Lang(object):

    def __init__(self):
        self.localedir: Optional[str] = None
        self.do_setup: bool = False
        self.used_lang: Optional[str] = None
        self.ignore = os.getenv("IGNORE_GETTEXT", None)
        self.domains: Dict[str, Domain] = {}
        self.use_dummy: bool = False
        self.is_setup: bool = False
        return

    @staticmethod
    def dummy(text: str):
        return text

    def setup(self, localedir: str, used_lang: str = "en") -> bool:
        self.is_setup = False
        if localedir == "":
            return False

        if os.path.exists(localedir) is False:
            self.use_dummy = True
            return False

        self.localedir = localedir

        if self.used_lang is None:
            self.used_lang = used_lang

        if self.ignore is not None:
            self.use_dummy = True
        self.is_setup = True
        return True

    def _get_domain(self, domain_name: str) -> Optional[Domain]:

        try:
            _domain = self.domains[domain_name]
        except KeyError:
            return None

        return _domain

    def _create_domain(self, domain_name: str) -> Domain:
        _domain = Domain(localedir=self.localedir,
                         domain=domain_name,
                         use_dummy=self.use_dummy,
                         ignore=self.ignore,
                         used_lang=self.used_lang)

        self.domains[_domain.domain] = _domain
        return _domain

    def set_language(self, language: str):
        self.used_lang = language

        for _name in self.domains:
            _domain = self.domains[_name]
            _domain.is_set = False
            _domain.used_lang = self.used_lang
            _domain.create()
            _domain.load()
        return

    def add(self, domain_name: str, callback: Any = None):

        _domain = self._get_domain(domain_name)
        if _domain is None:
            _domain = self._create_domain(domain_name)
            _domain.create()

        _domain.callback.append(callback)
        _domain.load()
        return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import abc
from abc import ABCMeta
from typing import List

from datetime import datetime

__all__ = [
    "Message",
    "Timer",
    "Progress",
    "Writer"
]


class Message(object):

    def __init__(self, **kwargs):
        self.time = datetime.now()
        self.app: str = ""
        self.tag: str = ""
        self.content: str = ""
        self.level: str = ""
        self.raw: bool = False

        # noinspection PyTypeChecker
        self.progress: Progress = None

        item = kwargs.get("app", None)
        if item is not None:
            self.app = item

        item = kwargs.get("tag", None)
        if item is not None:
            self.tag = item

        item = kwargs.get("level", None)
        if item is not None:
            self.level = item

        item = kwargs.get("content", None)
        if item is not None:
            self.content = item

        item = kwargs.get("raw", None)
        if item is not None:
            self.raw = item

        item = kwargs.get("progress", None)
        if item is not None:
            self.progress = item
        return


class Timer(object):

    def __init__(self, content: str, append_callback):
        self.content: str = content
        self.start: datetime = datetime.now()
        self._append = append_callback
        return

    def stop(self):
        delta = datetime.now() - self.start
        content = "Runtime: {0:s} for {1:s}".format(str(delta), self.content)

        _message = Message(content=content, level="TIMER")
        self._append(_message)
        return


class Progress(object):

    def __init__(self, limit: int, interval: int, append_callback):
        self.limit: int = limit
        self.counter: int = 0
        self.value: float = 0.0
        self.finished: bool = False
        self.interval: int = interval
        self.interval_counter: int = 0
        self.append = append_callback
        self.length = len(str(self.limit))
        return

    def _recalc(self):
        """recalculate progress.
        """
        self.value = float(self.counter) * 100.0 / float(self.limit)

        if self.interval != 0:
            self.interval_counter += 1
            if self.interval_counter == self.interval:
                self.interval_counter = 0

        if self.counter == self.limit:
            self.finished = True
        return

    def set(self, value: int):
        self.counter = value
        self._recalc()

        if self.interval_counter != 0:
            return

        _message = Message(level="PROGRESS", progress=self)
        self.append(_message)
        return

    def inc(self):
        self.counter += 1

        self._recalc()

        if self.interval_counter != 0:
            return

        _message = Message(level="PROGRESS", progress=self)
        self.append(_message)
        return

    def dec(self):
        self.counter -= 1

        self._recalc()

        if self.interval_counter != 0:
            return

        _message = Message(level="PROGRESS", progress=self)
        self.append(_message)
        return


class Writer(metaclass=ABCMeta):

    def __init__(self, name: str, index: List[str]):
        self.id = name
        self.index = index
        return

    @abc.abstractmethod
    def setup(self, **kwargs):  # pragma: no cover
        return

    @abc.abstractmethod
    def write(self, item: Message):  # pragma: no cover
        return

    @abc.abstractmethod
    def clear(self) -> bool:  # pragma: no cover
        return True

    @abc.abstractmethod
    def open(self) -> bool:  # pragma: no cover
        return True

    @abc.abstractmethod
    def close(self) -> bool:  # pragma: no cover
        return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import sqlite3

from dataclasses import dataclass
from multiprocessing import Lock
from typing import Optional

import bbutil

__all__ = [
    "Connection"
]


@dataclass
class Connection(object):

    use_memory: bool = False
    filename: str = ""

    _lock: Optional[Lock] = None
    _connection: Optional[sqlite3.Connection] = None
    _cursor: Optional[sqlite3.Cursor] = None

    @property
    def connection(self) -> Optional[sqlite3.Connection]:
        return self._connection

    def cursor(self) -> Optional[sqlite3.Cursor]:
        if self._connection is not None:
            return self._connection.cursor()
        return None

    def setup(self, **kwargs):
        if self._lock is None:
            self._lock = Lock()

        _value = kwargs.get("use_memory", None)
        if _value is not None:
            self.use_memory = _value

        _value = kwargs.get("filename", None)
        if _value is not None:
            self.filename = _value
        return

    def _connect_memory(self) -> bool:
        try:
            self._connection = sqlite3.connect(':memory:',
                                               detect_types=sqlite3.PARSE_DECLTYPES | sqlite3.PARSE_COLNAMES)
        except sqlite3.OperationalError as e:
            self._connection = None
            bbutil.log.error("Unable to create database in memory!")
            bbutil.log.exception(e)
            return False

        self.filename = "memory"
        return True

    def _connect_file(self) -> bool:
        try:
            self._connection = sqlite3.connect(self.filename,
                                               detect_types=sqlite3.PARSE_DECLTYPES | sqlite3.PARSE_COLNAMES)
        except sqlite3.OperationalError as e:
            bbutil.log.error("Unable to create database: {0:s}".format(self.filename))
            bbutil.log.exception(e)
            return False

        bbutil.log.debug1("SQLite3", "Connect to {0:s}".format(self.filename))
        return True

    def connect(self) -> bool:
        self._lock.acquire()

        if self._connection is not None:
            bbutil.log.error("Connection is still active!")
            return False

        if self.use_memory is True:
            _check = self._connect_memory()
        else:
            _check = self._connect_file()

        if _check is False:
            self._lock.release()

        return _check

    def commit(self) -> bool:
        if self._connection is None:
            bbutil.log.error("No connection!")
            return False

        try:
            self._connection.commit()
        except sqlite3.OperationalError as e:
            bbutil.log.error("Unable to commit to database!")
            bbutil.log.exception(e)
            return False
        return True

    def abort(self):
        if self._lock is None:
            return
        self._lock.release()
        return

    def reset(self):
        self._lock = Lock()
        self._connection = None
        return

    def release(self) -> bool:
        if self._connection is None:
            bbutil.log.error("No connection!")
            return False

        if self._lock is None:
            bbutil.log.error("No valid lock!")
            return False

        try:
            self._connection.close()
        except sqlite3.OperationalError as e:
            bbutil.log.error("Unable to close connection!")
            bbutil.log.exception(e)
            return False

        self._connection = None

        if self.use_memory is False:
            bbutil.log.debug1("SQLite3", "Close {0:s}".format(self.filename))

        self._lock.release()
        return True



#!/usr/bin/python3
# coding=utf-8

# Copyright (C) 2020, Siemens Healthcare Diagnostics Products GmbH
# Licensed under the Siemens Inner Source License 1.2, see LICENSE.md.

import os

from bbutil.utils import full_path, fix_sep


__all__ = [
    "find_data_files"
]


def _find_folders(path: str, folders: list, exlude_list: list):
    for root, dirs, files in os.walk(path, topdown=True):
        for name in sorted(dirs):

            _item = full_path("{0:s}/{1:s}".format(root, name))

            do_continue = False

            for _exclude in exlude_list:
                if _exclude in _item:
                    do_continue = True

            if do_continue is True:
                continue

            if "__pycache__" in _item:
                continue

            folders.append(_item)
    return


def _find_files(path: str, current_path: str, exlude_list: list) -> list:
    _files = []
    for _item in sorted(os.listdir(path)):

        _fullname = full_path("{0:s}/{1:s}".format(path, _item))
        _path = _fullname.replace(current_path, "")

        if "__pycache__" in _fullname:
            continue

        do_continue = False

        for _exclude in exlude_list:
            if _exclude in _item:
                do_continue = True

        if do_continue is True:
            continue

        if os.path.isdir(_fullname) is True:
            continue

        _path = fix_sep(_path)
        _files.append(_path)
    return _files


def find_data_files(folder: str, target: str, package_files: list, exlude_list: list):
    current_path = "{0:s}{1:s}".format(os.getcwd(), os.sep)
    target_folder = full_path("{0:s}/{1:s}".format(current_path, folder))

    _folders = [
        target_folder
    ]

    _find_folders(target_folder, _folders, exlude_list)

    for _item in _folders:
        _files = _find_files(_item, current_path, exlude_list)

        _target_folder = "{0:s}{1:s}".format(target_folder, os.sep)
        _name = _item.replace(_target_folder, "")

        if _name == target_folder:
            _name = target
        else:
            _name = "{0:s}/{1:s}".format(target, _name)
            _name = fix_sep(_name)

        _count = len(_files)
        if _count == 0:
            continue

        package_data = (_name, _files)
        package_files.append(package_data)
    return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

from typing import List, Optional

from PySide6.QtCore import Qt
from PySide6.QtGui import QColor, QFont, QTextCursor
from PySide6.QtWidgets import QTextEdit, QProgressBar

from bbutil.logging.types import Writer, Message
from bbutil.utils import get_terminal_size

__all__ = [
    "QtWriter"
]

classname = "QtWriter"
_index = ["INFORM", "DEBUG1", "DEBUG2", "DEBUG3", "WARN", "ERROR", "EXCEPTION", "TIMER", "PROGRESS"]

_schemes = {
    "NORMAL": QColor(Qt.black),
    "INFORM": QColor(Qt.darkGreen),
    "DEBUG1": QColor(Qt.cyan),
    "DEBUG2": QColor(Qt.cyan),
    "DEBUG3": QColor(Qt.cyan),
    "WARN": QColor(Qt.magenta),
    "ERROR": QColor(Qt.red),
    "EXCEPTION": QColor(Qt.red),
    "TIMER": QColor(Qt.yellow)
}


class QtWriter(Writer):

    def __init__(self):
        Writer.__init__(self, "TkLogger", _index)

        self.text_control: Optional[QTextEdit] = None
        self.progress_control: Optional[QProgressBar] = None

        self.progress_reset: bool = False
        self.encoding: str = ""
        self.text_space: int = 15
        self.text_size: int = 11
        self.seperator: str = "|"
        self.length: int = 0
        self.error_index: List[str] = ["ERROR", "EXCEPTION"]

        size_x, size_y = get_terminal_size()

        self.line_width: int = size_x
        self.bar_len: int = 50
        self.active: bool = False

        self._prepared: bool = False
        self._buffer: List[Message] = []
        return

    @property
    def _is_ready(self) -> bool:
        if (self.text_control is None) or (self.progress_control is None):
            return False
        return True

    def setup(self, **kwargs):
        item = kwargs.get("text_space", None)
        if item is not None:
            self.text_space = item

        item = kwargs.get("seperator", None)
        if item is not None:
            self.seperator = item

        item = kwargs.get("error_index", None)
        if item is not None:
            self.error_index = item

        item = kwargs.get("bar_len", None)
        if item is not None:
            self.bar_len = item

        item = kwargs.get("text_size", None)
        if item is not None:
            self.text_size = item

        item = kwargs.get("text_control", None)
        if item is not None:
            self.text_control = item
            self._set_text()

        item = kwargs.get("progress_control", None)
        if item is not None:
            self.progress_control = item
        return

    def _set_text(self):
        if self.text_control is None:
            return

        font = QFont("Monospace")
        font.setStyleHint(QFont.TypeWriter)
        font.setPointSize(self.text_size)

        self.text_control.setFont(font)
        return

    def open(self) -> bool:
        self._init_progress()
        self.active = True
        return True

    def close(self) -> bool:  # pragma: no cover
        self.active = False
        return True

    def clear(self) -> bool:
        self.progress_control.setValue(0)
        self.progress_control.reset()
        self.progress_reset = True
        return True

    def _init_progress(self):
        if self.progress_control is None:
            return
        self.progress_control.setMinimum(0)
        self.progress_control.setMaximum(100)
        self.progress_reset = False
        return

    def _set_progress(self, item: Message):
        if self.progress_reset is True:
            self._init_progress()
        value = int(float(item.progress.counter * 100) / float(item.progress.limit))
        self.progress_control.setValue(value)
        return

    def _write(self, item: Message):
        if item.level == "":
            item.level = "NORMAL"

        _color = _schemes[item.level]

        if item.tag == "":
            text = "{0:s}\n".format(item.content)

            self.text_control.setTextColor(_color)
            self.text_control.insertPlainText(text)
            self.text_control.setTextColor(QColor(Qt.black))
        else:
            tag = item.tag.ljust(self.text_space)
            text = "{0:s} {1:s}\n".format(self.seperator, item.content)

            self.text_control.setTextColor(_color)
            self.text_control.insertPlainText(tag)

            self.text_control.setTextColor(QColor(Qt.black))
            self.text_control.insertPlainText(text)

        self.text_control.moveCursor(QTextCursor.End)
        return

    def write(self, item: Message):
        if item.level == "PROGRESS":
            self._set_progress(item)
            return

        if self.active is False:
            return

        if self._is_ready is False:
            self._buffer.append(item)
            return

        for _item in self._buffer:
            self._write(_item)

        self._buffer.clear()

        self._write(item)
        return



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

import abc
from abc import ABCMeta
from dataclasses import dataclass, field
from typing import Optional, List

import bbutil

from bbutil.database.sqlite import SQLite
from bbutil.database.table import Table

__all__ = [
    "Database"
]


@dataclass
class Database(metaclass=ABCMeta):

    name: str = ""
    sqlite: Optional[SQLite] = None
    tables: List[Table] = field(default_factory=list)
    filename: str = ""

    @abc.abstractmethod
    def init(self):
        pass

    @abc.abstractmethod
    def prepare(self, **kwargs) -> bool:
        pass

    @abc.abstractmethod
    def clear_data(self):
        pass

    def get_table(self, name: str) -> Optional[Table]:
        for _table in self.tables:
            if _table.name == name:
                return _table
        return None

    def clear(self):
        for _table in self.tables:
            _table.clear()

        self.clear_data()
        return

    def store(self) -> int:
        _count = 0
        for _table in self.tables:
            _count += _table.store()
        return _count

    def load(self) -> int:
        _count = 0
        for _table in self.tables:
            _count += _table.load()
        return _count

    def start(self) -> bool:
        self.init()

        if (self.name == "") or (self.filename == ""):
            bbutil.log.error("File- or database-name is missing!")
            return False

        self.sqlite = SQLite(name=self.name, filename=self.filename)

        self.sqlite.prepare()

        _check = self.prepare()
        if _check is False:
            bbutil.log.error("Preparation of tables has failed!")
            return False

        for _table in self.tables:
            _check = _table.init()
            if _check is False:
                return False
        return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

from typing import List
from dataclasses import dataclass, field

import bbutil

from bbutil.worker import Worker

__all__ = [
    "CallManager",
    "Worker01",
    "Worker02"
]


@dataclass
class CallManager(object):

    start: int = 0
    stop: int = 0
    prepare: int = 0
    run: int = 0
    close: int = 0
    abort: int = 0

    def count(self, name: str):
        _value = getattr(self, name)
        _value += 1
        setattr(self, name, _value)
        return

    def setup(self, worker: Worker):
        worker.set_callback(start=lambda: self.count("start"))
        worker.set_callback(stop=lambda: self.count("stop"))
        worker.set_callback(prepare=lambda: self.count("prepare"))
        worker.set_callback(run=lambda: self.count("run"))
        worker.set_callback(close=lambda: self.count("close"))
        worker.set_callback(abort=lambda: self.count("abort"))
        return

    def info(self):
        bbutil.log.inform("start", "{0:d}".format(self.start))
        bbutil.log.inform("stop", "{0:d}".format(self.stop))
        bbutil.log.inform("prepare", "{0:d}".format(self.prepare))
        bbutil.log.inform("run", "{0:d}".format(self.run))
        bbutil.log.inform("close", "{0:d}".format(self.close))
        bbutil.log.inform("abort", "{0:d}".format(self.abort))
        return


@dataclass
class Worker01(Worker):

    exit_prepare: bool = True
    exit_run: bool = True
    exit_close: bool = True

    def init(self):
        self.set_id("Worker01")
        return

    def prepare(self) -> bool:
        return self.exit_prepare

    def run(self) -> bool:
        return self.exit_run

    def close(self) -> bool:
        return self.exit_close


@dataclass
class Worker02(Worker):

    max: int = 50000
    iterate_list: List[int] = field(default_factory=list)

    def init(self):
        self.set_id("Worker02")
        return

    def prepare(self) -> bool:
        _max = self.max
        _range = range(0, _max)
        _progress = bbutil.log.progress(_max)

        for n in _range:
            self.iterate_list.append(n)
            _progress.inc()

        bbutil.log.clear()
        return True

    def run(self) -> bool:
        _max = len(self.iterate_list)
        _progress = bbutil.log.progress(_max)

        n = 0
        for x in self.iterate_list:
            self.iterate_list[n] = x + 1
            _progress.inc()
            n += 1

        bbutil.log.clear()
        return True

    def close(self) -> bool:
        _max = len(self.iterate_list)
        _progress = bbutil.log.progress(_max)

        n = 0
        for x in self.iterate_list:
            self.iterate_list[n] = x - 1
            _progress.inc()
            n += 1

        bbutil.log.clear()
        return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#    Copyright (C) 2017, Kai Raphahn <kai.raphahn@laburec.de>
#

__all__ = [
    "testdata1",
    "testdata2"
]


testdata1 = [
    (
       "data",
       [
          "testdata/testos1.py",
          "testdata/testos2.py",
          "testdata/testos3.py"
       ]
    ),
    (
       "data/database",
       [
           "testdata/database/test_add_columns.sqlite",
           "testdata/database/test_bulk.sqlite",
           "testdata/database/test_check_table.sqlite",
           "testdata/database/test_database.sqlite",
           "testdata/database/test_select.sqlite",
           "testdata/database/test_update.sqlite"
       ]
    ),
    (
       "data/testlang",
       [
          "testdata/testlang/__init__.py"
       ]
    ),
    (
       "data/testlang/test1",
       [
          "testdata/testlang/test1/__init__.py",
          "testdata/testlang/test1/tester.py"
       ]
    ),
    (
       "data/testlang/test2",
       [
          "testdata/testlang/test2/__init__.py"
       ]
    )
]

testdata2 = [
    (
        'data/database',
        [
            "testdata/database/test_add_columns.sqlite",
            "testdata/database/test_bulk.sqlite",
            "testdata/database/test_check_table.sqlite",
            "testdata/database/test_database.sqlite",
            "testdata/database/test_select.sqlite",
            "testdata/database/test_update.sqlite"
        ]
    )
]
